@@ -1,3 +1,4 @@
+{-# LANGUAGE NoImplicitPrelude #-}
 {-# LANGUAGE RecordWildCards, DeriveDataTypeable, OverloadedStrings #-}
 
 -- | Nix configuration
@@ -7,23 +8,16 @@ module Stack.Config.Nix
        ,StackNixException(..)
        ) where
 
-import Control.Monad (when)
-import Data.Maybe
-import Data.Monoid.Extra
+import Stack.Prelude
 import qualified Data.Text as T
-import Data.Typeable
 import Distribution.System (OS (..))
 import Stack.Types.Version
 import Stack.Types.Nix
 import Stack.Types.Compiler
-import Stack.Types.StringError
-import Control.Exception.Lifted
-import Control.Monad.Catch (throwM,MonadCatch)
-import Prelude
 
 -- | Interprets NixOptsMonoid options.
 nixOptsFromMonoid
-    :: (Monad m, MonadCatch m)
+    :: MonadUnliftIO m
     => NixOptsMonoid
     -> OS
     -> m NixOpts
@@ -39,15 +33,16 @@ nixOptsFromMonoid NixOptsMonoid{..} os = do
                           ++ prefixAll (T.pack "-I") (fromFirst [] nixMonoidPath)
         nixAddGCRoots   = fromFirst False nixMonoidAddGCRoots
     when (not (null nixPackages) && isJust nixInitFile) $
-       throwM NixCannotUseShellFileAndPackagesException
+       throwIO NixCannotUseShellFileAndPackagesException
     return NixOpts{..}
   where prefixAll p (x:xs) = p : x : prefixAll p xs
         prefixAll _ _      = []
 
-nixCompiler :: CompilerVersion -> T.Text
+nixCompiler :: CompilerVersion a -> Either StringException T.Text
 nixCompiler compilerVersion =
   let -- These are the latest minor versions for each respective major version available in nixpkgs
-      fixMinor "8.0" = "8.0.1"
+      fixMinor "8.2" = "8.2.1"
+      fixMinor "8.0" = "8.0.2"
       fixMinor "7.10" = "7.10.3"
       fixMinor "7.8" = "7.8.4"
       fixMinor "7.6" = "7.6.3"
@@ -60,8 +55,8 @@ nixCompiler compilerVersion =
                                           (T.filter (/= '.')
                                              (fixMinor (versionText v)))
   in case compilerVersion of
-       GhcVersion v -> nixCompilerFromVersion v
-       _ -> errorString "Only GHC is supported by stack --nix"
+       GhcVersion v -> Right $ nixCompilerFromVersion v
+       _ -> Left $ stringException "Only GHC is supported by stack --nix"
 
 -- Exceptions thown specifically by Stack.Nix
 data StackNixException
