@@ -1,3 +1,4 @@
+{-# LANGUAGE NoImplicitPrelude #-}
 {-# LANGUAGE ConstraintKinds       #-}
 {-# LANGUAGE FlexibleContexts      #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
@@ -16,35 +17,20 @@ module Stack.Coverage
     , generateHpcMarkupIndex
     ) where
 
-import           Control.Exception.Safe (handleIO)
-import           Control.Exception.Lifted
-import           Control.Monad (liftM, when, unless, void, (<=<))
-import           Control.Monad.IO.Class
-import           Control.Monad.Logger
-import           Control.Monad.Trans.Resource
+import           Stack.Prelude
 import qualified Data.ByteString.Char8 as S8
-import           Data.Foldable (forM_, asum, toList)
-import           Data.Function
 import           Data.List
 import qualified Data.Map.Strict as Map
-import           Data.Maybe
-import           Data.Maybe.Extra (mapMaybeM)
-import           Data.Monoid ((<>))
-import           Data.String
-import           Data.Text (Text)
 import qualified Data.Text as T
 import qualified Data.Text.Encoding as T
 import qualified Data.Text.IO as T
 import qualified Data.Text.Lazy as LT
-import           Data.Traversable (forM)
 import           Path
 import           Path.Extra (toFilePathNoTrailingSep)
 import           Path.IO
-import           Prelude hiding (FilePath, writeFile)
-import           Stack.Build.Source (parseTargetsFromBuildOpts)
 import           Stack.Build.Target
 import           Stack.Config (getLocalPackages)
-import           Stack.Constants
+import           Stack.Constants.Config
 import           Stack.Package
 import           Stack.PrettyPrint
 import           Stack.Types.Compiler
@@ -52,8 +38,7 @@ import           Stack.Types.Config
 import           Stack.Types.Package
 import           Stack.Types.PackageIdentifier
 import           Stack.Types.PackageName
-import           Stack.Types.StackT (StackM)
-import           Stack.Types.StringError
+import           Stack.Types.Runner
 import           Stack.Types.Version
 import           System.FilePath (isPathSeparator)
 import           System.Process.Read
@@ -62,38 +47,35 @@ import           Trace.Hpc.Tix
 import           Web.Browser (openBrowser)
 
 -- | Invoked at the beginning of running with "--coverage"
-deleteHpcReports :: (StackM env m, HasEnvConfig env)
-                 => m ()
+deleteHpcReports :: HasEnvConfig env => RIO env ()
 deleteHpcReports = do
     hpcDir <- hpcReportDir
-    ignoringAbsence (removeDirRecur hpcDir)
+    liftIO $ ignoringAbsence (removeDirRecur hpcDir)
 
 -- | Move a tix file into a sub-directory of the hpc report directory. Deletes the old one if one is
 -- present.
-updateTixFile :: (StackM env m, HasEnvConfig env)
-              => PackageName -> Path Abs File -> String -> m ()
+updateTixFile :: HasEnvConfig env => PackageName -> Path Abs File -> String -> RIO env ()
 updateTixFile pkgName tixSrc testName = do
     exists <- doesFileExist tixSrc
     when exists $ do
         tixDest <- tixFilePath pkgName testName
-        ignoringAbsence (removeFile tixDest)
+        liftIO $ ignoringAbsence (removeFile tixDest)
         ensureDir (parent tixDest)
         -- Remove exe modules because they are problematic. This could be revisited if there's a GHC
         -- version that fixes https://ghc.haskell.org/trac/ghc/ticket/1853
         mtix <- readTixOrLog tixSrc
         case mtix of
-            Nothing -> $logError $ "Failed to read " <> T.pack (toFilePath tixSrc)
+            Nothing -> logError $ "Failed to read " <> T.pack (toFilePath tixSrc)
             Just tix -> do
                 liftIO $ writeTix (toFilePath tixDest) (removeExeModules tix)
                 -- TODO: ideally we'd do a file move, but IIRC this can
                 -- have problems. Something about moving between drives
                 -- on windows?
                 copyFile tixSrc =<< parseAbsFile (toFilePath tixDest ++ ".premunging")
-                ignoringAbsence (removeFile tixSrc)
+                liftIO $ ignoringAbsence (removeFile tixSrc)
 
 -- | Get the directory used for hpc reports for the given pkgId.
-hpcPkgPath :: (StackM env m, HasEnvConfig env)
-            => PackageName -> m (Path Abs Dir)
+hpcPkgPath :: HasEnvConfig env => PackageName -> RIO env (Path Abs Dir)
 hpcPkgPath pkgName = do
     outputDir <- hpcReportDir
     pkgNameRel <- parseRelDir (packageNameString pkgName)
@@ -101,16 +83,16 @@ hpcPkgPath pkgName = do
 
 -- | Get the tix file location, given the name of the file (without extension), and the package
 -- identifier string.
-tixFilePath :: (StackM env m, HasEnvConfig env)
-            => PackageName -> String ->  m (Path Abs File)
+tixFilePath :: HasEnvConfig env
+            => PackageName -> String -> RIO env (Path Abs File)
 tixFilePath pkgName testName = do
     pkgPath <- hpcPkgPath pkgName
     tixRel <- parseRelFile (testName ++ "/" ++ testName ++ ".tix")
     return (pkgPath </> tixRel)
 
 -- | Generates the HTML coverage report and shows a textual coverage summary for a package.
-generateHpcReport :: (StackM env m, HasEnvConfig env)
-                  => Path Abs Dir -> Package -> [Text] -> m ()
+generateHpcReport :: HasEnvConfig env
+                  => Path Abs Dir -> Package -> [Text] -> RIO env ()
 generateHpcReport pkgDir package tests = do
     compilerVersion <- view actualCompilerVersionL
     -- If we're using > GHC 7.10, the hpc 'include' parameter must specify a ghc package key. See
@@ -118,11 +100,15 @@ generateHpcReport pkgDir package tests = do
     let pkgName = packageNameText (packageName package)
         pkgId = packageIdentifierString (packageIdentifier package)
         ghcVersion = getGhcVersion compilerVersion
+        hasLibrary =
+          case packageLibraries package of
+            NoLibraries -> False
+            HasLibraries _ -> True
     eincludeName <-
         -- Pre-7.8 uses plain PKG-version in tix files.
         if ghcVersion < $(mkVersion "7.10") then return $ Right $ Just pkgId
         -- We don't expect to find a package key if there is no library.
-        else if not (packageHasLibrary package) then return $ Right Nothing
+        else if not hasLibrary then return $ Right Nothing
         -- Look in the inplace DB for the package key.
         -- See https://github.com/commercialhaskell/stack/issues/1181#issuecomment-148968986
         else do
@@ -132,7 +118,7 @@ generateHpcReport pkgDir package tests = do
             eincludeName <- findPackageFieldForBuiltPackage pkgDir (packageIdentifier package) hpcNameField
             case eincludeName of
                 Left err -> do
-                    $logError err
+                    logError err
                     return $ Left err
                 Right includeName -> return $ Right $ Just $ T.unpack includeName
     forM_ tests $ \testName -> do
@@ -148,16 +134,17 @@ generateHpcReport pkgDir package tests = do
                         Just includeName -> ["--include", includeName ++ ":"]
                         Nothing -> []
                 mreportPath <- generateHpcReportInternal tixSrc reportDir report extraArgs extraArgs
-                forM_ mreportPath (displayReportPath report)
+                forM_ mreportPath (displayReportPath report . display)
 
-generateHpcReportInternal :: (StackM env m, HasEnvConfig env)
-                          => Path Abs File -> Path Abs Dir -> Text -> [String] -> [String] -> m (Maybe (Path Abs File))
+generateHpcReportInternal :: HasEnvConfig env
+                          => Path Abs File -> Path Abs Dir -> Text -> [String] -> [String]
+                          -> RIO env (Maybe (Path Abs File))
 generateHpcReportInternal tixSrc reportDir report extraMarkupArgs extraReportArgs = do
     -- If a .tix file exists, move it to the HPC output directory and generate a report for it.
     tixFileExists <- doesFileExist tixSrc
     if not tixFileExists
         then do
-            $logError $ T.concat
+            logError $ T.concat
                  [ "Didn't find .tix for "
                  , report
                  , " - expected to find it at "
@@ -167,21 +154,21 @@ generateHpcReportInternal tixSrc reportDir report extraMarkupArgs extraReportArg
             return Nothing
         else (`catch` \err -> do
                  let msg = show (err :: ReadProcessException)
-                 $logError (T.pack msg)
+                 logError (T.pack msg)
                  generateHpcErrorReport reportDir $ sanitize msg
                  return Nothing) $
-             (`onException` $logError ("Error occurred while producing " <> report)) $ do
+             (`onException` logError ("Error occurred while producing " <> report)) $ do
             -- Directories for .mix files.
             hpcRelDir <- hpcRelativeDir
             -- Compute arguments used for both "hpc markup" and "hpc report".
-            pkgDirs <- liftM Map.keys getLocalPackages
+            pkgDirs <- liftM (map lpvRoot . Map.elems . lpProject) getLocalPackages
             let args =
                     -- Use index files from all packages (allows cross-package coverage results).
                     concatMap (\x -> ["--srcdir", toFilePathNoTrailingSep x]) pkgDirs ++
                     -- Look for index files in the correct dir (relative to each pkgdir).
                     ["--hpcdir", toFilePathNoTrailingSep hpcRelDir, "--reset-hpcdirs"]
             menv <- getMinimalEnvOverride
-            $logInfo $ "Generating " <> report
+            logInfo $ "Generating " <> report
             outputLines <- liftM (map (S8.filter (/= '\r')) . S8.lines) $
                 readProcessStdout Nothing menv "hpc"
                 ( "report"
@@ -202,13 +189,13 @@ generateHpcReportInternal tixSrc reportDir report extraMarkupArgs extraReportArg
                             , " the hpc program. Please report this issue if you think"
                             , " your coverage report should have meaningful results."
                             ]
-                    $logError (msg False)
+                    logError (msg False)
                     generateHpcErrorReport reportDir (msg True)
                     return Nothing
                 else do
                     let reportPath = reportDir </> $(mkRelFile "hpc_index.html")
                     -- Print output, stripping @\r@ characters because Windows.
-                    forM_ outputLines ($logInfo . T.decodeUtf8)
+                    forM_ outputLines (logInfo . T.decodeUtf8)
                     -- Generate the markup.
                     void $ readProcessStdout Nothing menv "hpc"
                         ( "markup"
@@ -225,8 +212,8 @@ data HpcReportOpts = HpcReportOpts
     , hroptsOpenBrowser :: Bool
     } deriving (Show)
 
-generateHpcReportForTargets :: (StackM env m, HasEnvConfig env)
-                            => HpcReportOpts -> m ()
+generateHpcReportForTargets :: HasEnvConfig env
+                            => HpcReportOpts -> RIO env ()
 generateHpcReportForTargets opts = do
     let (tixFiles, targetNames) = partition (".tix" `T.isSuffixOf`) (hroptsInputs opts)
     targetTixFiles <-
@@ -236,20 +223,18 @@ generateHpcReportForTargets opts = do
          then return []
          else do
              when (hroptsAll opts && not (null targetNames)) $
-                 $logWarn $ "Since --all is used, it is redundant to specify these targets: " <> T.pack (show targetNames)
-             (_,_,targets) <- parseTargetsFromBuildOpts
+                 logWarn $ "Since --all is used, it is redundant to specify these targets: " <> T.pack (show targetNames)
+             (_,_,targets) <- parseTargets
                  AllowNoTargets
                  defaultBuildOptsCLI
                     { boptsCLITargets = if hroptsAll opts then [] else targetNames }
              liftM concat $ forM (Map.toList targets) $ \(name, target) ->
                  case target of
-                     STUnknown -> throwString $
-                         "Error: " ++ packageNameString name ++ " isn't a known local page"
-                     STNonLocal -> throwString $
+                     TargetAll Dependency -> throwString $
                          "Error: Expected a local package, but " ++
                          packageNameString name ++
                          " is either an extra-dep or in the snapshot."
-                     STLocalComps comps -> do
+                     TargetComps comps -> do
                          pkgPath <- hpcPkgPath name
                          forM (toList comps) $ \nc ->
                              case nc of
@@ -259,7 +244,7 @@ generateHpcReportForTargets opts = do
                                      "Can't specify anything except test-suites as hpc report targets (" ++
                                      packageNameString name ++
                                      " is used with a non test-suite target)"
-                     STLocalAll -> do
+                     TargetAll ProjectPackage -> do
                          pkgPath <- hpcPkgPath name
                          exists <- doesDirExist pkgPath
                          if exists
@@ -284,12 +269,11 @@ generateHpcReportForTargets opts = do
     forM_ mreportPath $ \reportPath ->
         if hroptsOpenBrowser opts
             then do
-                $prettyInfo $ "Opening" <+> display reportPath <+> "in the browser."
+                prettyInfo $ "Opening" <+> display reportPath <+> "in the browser."
                 void $ liftIO $ openBrowser (toFilePath reportPath)
-            else displayReportPath report reportPath
+            else displayReportPath report (display reportPath)
 
-generateHpcUnifiedReport :: (StackM env m, HasEnvConfig env)
-                         => m ()
+generateHpcUnifiedReport :: HasEnvConfig env => RIO env ()
 generateHpcUnifiedReport = do
     outputDir <- hpcReportDir
     ensureDir outputDir
@@ -303,7 +287,7 @@ generateHpcUnifiedReport = do
     let tixFiles = tixFiles0  ++ extraTixFiles
         reportDir = outputDir </> $(mkRelDir "combined/all")
     if length tixFiles < 2
-        then $logInfo $ T.concat
+        then logInfo $ T.concat
             [ if null tixFiles then "No tix files" else "Only one tix file"
             , " found in "
             , T.pack (toFilePath outputDir)
@@ -312,14 +296,15 @@ generateHpcUnifiedReport = do
         else do
             let report = "unified report"
             mreportPath <- generateUnionReport report reportDir tixFiles
-            forM_ mreportPath (displayReportPath report)
+            forM_ mreportPath (displayReportPath report . display)
 
-generateUnionReport :: (StackM env m, HasEnvConfig env)
-                    => Text -> Path Abs Dir -> [Path Abs File] -> m (Maybe (Path Abs File))
+generateUnionReport :: HasEnvConfig env
+                    => Text -> Path Abs Dir -> [Path Abs File]
+                    -> RIO env (Maybe (Path Abs File))
 generateUnionReport report reportDir tixFiles = do
     (errs, tix) <- fmap (unionTixes . map removeExeModules) (mapMaybeM readTixOrLog tixFiles)
-    $logDebug $ "Using the following tix files: " <> T.pack (show tixFiles)
-    unless (null errs) $ $logWarn $ T.concat $
+    logDebug $ "Using the following tix files: " <> T.pack (show tixFiles)
+    unless (null errs) $ logWarn $ T.concat $
         "The following modules are left out of the " : report : " due to version mismatches: " :
         intersperse ", " (map T.pack errs)
     tixDest <- liftM (reportDir </>) $ parseRelFile (dirnameString reportDir ++ ".tix")
@@ -327,13 +312,13 @@ generateUnionReport report reportDir tixFiles = do
     liftIO $ writeTix (toFilePath tixDest) tix
     generateHpcReportInternal tixDest reportDir report [] []
 
-readTixOrLog :: (MonadLogger m, MonadIO m, MonadBaseControl IO m) => Path b File -> m (Maybe Tix)
+readTixOrLog :: (MonadLogger m, MonadUnliftIO m) => Path b File -> m (Maybe Tix)
 readTixOrLog path = do
-    mtix <- liftIO (readTix (toFilePath path)) `catch` \errorCall -> do
-        $logError $ "Error while reading tix: " <> T.pack (show (errorCall :: ErrorCall))
+    mtix <- liftIO (readTix (toFilePath path)) `catchAny` \errorCall -> do
+        logError $ "Error while reading tix: " <> T.pack (show errorCall)
         return Nothing
     when (isNothing mtix) $
-        $logError $ "Failed to read tix file " <> T.pack (toFilePath path)
+        logError $ "Failed to read tix file " <> T.pack (toFilePath path)
     return mtix
 
 -- | Module names which contain '/' have a package name, and so they weren't built into the
@@ -351,8 +336,7 @@ unionTixes tixes = (Map.keys errs, Tix (Map.elems outputs))
         | hash1 == hash2 && len1 == len2 = Right (TixModule k hash1 len1 (zipWith (+) tix1 tix2))
     merge _ _ = Left ()
 
-generateHpcMarkupIndex :: (StackM env m, HasEnvConfig env)
-                       => m ()
+generateHpcMarkupIndex :: HasEnvConfig env => RIO env ()
 generateHpcMarkupIndex = do
     outputDir <- hpcReportDir
     let outputFile = outputDir </> $(mkRelFile "index.html")
@@ -364,7 +348,7 @@ generateHpcMarkupIndex = do
             let indexPath = subdir </> $(mkRelFile "hpc_index.html")
             exists' <- doesFileExist indexPath
             if not exists' then return Nothing else do
-                relPath <- stripDir outputDir indexPath
+                relPath <- stripProperPrefix outputDir indexPath
                 let package = dirname dir
                     testsuite = dirname subdir
                 return $ Just $ T.concat
@@ -402,7 +386,7 @@ generateHpcMarkupIndex = do
                 ["</tbody></table>"]) ++
         ["</body></html>"]
     unless (null rows) $
-        $logInfo $ "\nAn index of the generated HTML coverage reports is available at " <>
+        logInfo $ "\nAn index of the generated HTML coverage reports is available at " <>
             T.pack (toFilePath outputFile)
 
 generateHpcErrorReport :: MonadIO m => Path Abs Dir -> Text -> m ()
@@ -427,8 +411,9 @@ dirnameString :: Path r Dir -> String
 dirnameString = dropWhileEnd isPathSeparator . toFilePath . dirname
 
 findPackageFieldForBuiltPackage
-    :: (StackM env m, HasEnvConfig env)
-    => Path Abs Dir -> PackageIdentifier -> Text -> m (Either Text Text)
+    :: HasEnvConfig env
+    => Path Abs Dir -> PackageIdentifier -> Text
+    -> RIO env (Either Text Text)
 findPackageFieldForBuiltPackage pkgDir pkgId field = do
     distDir <- distDirFromDir pkgDir
     let inplaceDir = distDir </> $(mkRelDir "package.conf.inplace")
@@ -443,14 +428,14 @@ findPackageFieldForBuiltPackage pkgDir pkgId field = do
     if cabalVer < $(mkVersion "1.24")
         then do
             path <- liftM (inplaceDir </>) $ parseRelFile (pkgIdStr ++ "-inplace.conf")
-            $logDebug $ "Parsing config in Cabal < 1.24 location: " <> T.pack (toFilePath path)
+            logDebug $ "Parsing config in Cabal < 1.24 location: " <> T.pack (toFilePath path)
             exists <- doesFileExist path
             if exists then extractField path else notFoundErr
         else do
             -- With Cabal-1.24, it's in a different location.
-            $logDebug $ "Scanning " <> T.pack (toFilePath inplaceDir) <> " for files matching " <> T.pack pkgIdStr
+            logDebug $ "Scanning " <> T.pack (toFilePath inplaceDir) <> " for files matching " <> T.pack pkgIdStr
             (_, files) <- handleIO (const $ return ([], [])) $ listDir inplaceDir
-            $logDebug $ T.pack (show files)
+            logDebug $ T.pack (show files)
             case mapMaybe (\file -> fmap (const file) . (T.stripSuffix ".conf" <=< T.stripPrefix (T.pack (pkgIdStr ++ "-")))
                           . T.pack . toFilePath . filename $ file) files of
                 [] -> notFoundErr
@@ -458,12 +443,12 @@ findPackageFieldForBuiltPackage pkgDir pkgId field = do
                 _ -> return $ Left $ "Multiple files matching " <> T.pack (pkgIdStr ++ "-*.conf") <> " found in " <>
                     T.pack (toFilePath inplaceDir) <> ". Maybe try 'stack clean' on this package?"
 
-displayReportPath :: (StackM env m, HasAnsiAnn (Ann a), Display a)
-                  => Text -> a -> m ()
+displayReportPath :: (HasRunner env)
+                  => Text -> AnsiDoc -> RIO env ()
 displayReportPath report reportPath =
-     $prettyInfo $ "The" <+> fromString (T.unpack report) <+> "is available at" <+> display reportPath
+     prettyInfo $ "The" <+> fromString (T.unpack report) <+> "is available at" <+> reportPath
 
-findExtraTixFiles :: (StackM env m , HasEnvConfig env) => m [Path Abs File]
+findExtraTixFiles :: HasEnvConfig env => RIO env [Path Abs File]
 findExtraTixFiles = do
     outputDir <- hpcReportDir
     let dir = outputDir </> $(mkRelDir "extra-tix-files")
