@@ -1,3 +1,4 @@
+{-# LANGUAGE NoImplicitPrelude #-}
 {-# LANGUAGE CPP, ConstraintKinds, DeriveDataTypeable, FlexibleContexts, MultiWayIf, NamedFieldPuns,
              OverloadedStrings, PackageImports, RankNTypes, RecordWildCards, ScopedTypeVariables,
              TemplateHaskell, TupleSections #-}
@@ -20,15 +21,7 @@ module Stack.Docker
   ,StackDockerException(..)
   ) where
 
-import           Control.Applicative
-import           Control.Concurrent.MVar.Lifted (MVar,modifyMVar_,newMVar)
-import           Control.Exception.Lifted
-import           Control.Monad
-import           Control.Monad.Catch (MonadThrow,throwM,MonadCatch)
-import           Control.Monad.IO.Class (MonadIO,liftIO)
-import           Control.Monad.Logger (MonadLogger,logError,logInfo,logWarn)
-import           Control.Monad.Reader (MonadReader,runReaderT)
-import           Control.Monad.Trans.Control (MonadBaseControl)
+import           Stack.Prelude
 import           Control.Monad.Writer (execWriter,runWriter,tell)
 import qualified Crypto.Hash as Hash (Digest, MD5, hash)
 import           Data.Aeson.Extended (FromJSON(..),(.:),(.:?),(.!=),eitherDecode)
@@ -37,14 +30,11 @@ import qualified Data.ByteString.Char8 as BS
 import qualified Data.ByteString.Lazy.Char8 as LBS
 import           Data.Char (isSpace,toUpper,isAscii,isDigit)
 import           Data.Conduit.List (sinkNull)
-import           Data.List (dropWhileEnd,intercalate,isPrefixOf,isInfixOf,foldl')
+import           Data.List (dropWhileEnd,intercalate,isPrefixOf,isInfixOf)
 import           Data.List.Extra (trim, nubOrd)
-import           Data.Map.Strict (Map)
 import qualified Data.Map.Strict as Map
-import           Data.Maybe
 import           Data.Ord (Down(..))
 import           Data.Streaming.Process (ProcessExitedUnsuccessfully(..))
-import           Data.Text (Text)
 import qualified Data.Text as T
 import qualified Data.Text.Encoding as T
 import           Data.Time (UTCTime,LocalTime(..),diffDays,utcToLocalTime,getZonedTime,ZonedTime(..))
@@ -54,22 +44,21 @@ import           Path
 import           Path.Extra (toFilePathNoTrailingSep)
 import           Path.IO hiding (canonicalizePath)
 import qualified Paths_stack as Meta
-import           Prelude -- Fix redundant import warnings
 import           Stack.Config (getInContainer)
 import           Stack.Constants
+import           Stack.Constants.Config
 import           Stack.Docker.GlobalDB
 import           Stack.Types.PackageIndex
+import           Stack.Types.Runner
 import           Stack.Types.Version
 import           Stack.Types.Config
 import           Stack.Types.Docker
-import           Stack.Types.Internal
-import           Stack.Types.StackT
 import           Stack.Setup (ensureDockerStackExe)
 import           System.Directory (canonicalizePath,getHomeDirectory)
 import           System.Environment (getEnv,getEnvironment,getProgName,getArgs,getExecutablePath)
 import           System.Exit (exitSuccess, exitWith, ExitCode(..))
 import qualified System.FilePath as FP
-import           System.IO (stderr,stdin,stdout,hIsTerminalDevice, hClose)
+import           System.IO (stderr,stdin,stdout)
 import           System.IO.Error (isDoesNotExistError)
 import           System.IO.Unsafe (unsafePerformIO)
 import qualified System.PosixCompat.User as User
@@ -82,7 +71,6 @@ import           Text.Printf (printf)
 
 #ifndef WINDOWS
 import           Control.Concurrent (threadDelay)
-import qualified Control.Monad.Trans.Control as Control
 import           System.Posix.Signals
 import qualified System.Posix.User as PosixUser
 #endif
@@ -91,17 +79,17 @@ import qualified System.Posix.User as PosixUser
 -- Otherwise, runs the inner action.
 --
 -- This takes an optional release action which should be taken IFF control is
--- transfering away from the current process to the intra-container one.  The main use
+-- transferring away from the current process to the intra-container one.  The main use
 -- for this is releasing a lock.  After launching reexecution, the host process becomes
 -- nothing but an manager for the call into docker and thus may not hold the lock.
 reexecWithOptionalContainer
-    :: (StackM env m, HasConfig env)
+    :: HasConfig env
     => Maybe (Path Abs Dir)
-    -> Maybe (m ())
+    -> Maybe (RIO env ())
     -> IO ()
-    -> Maybe (m ())
-    -> Maybe (m ())
-    -> m ()
+    -> Maybe (RIO env ())
+    -> Maybe (RIO env ())
+    -> RIO env ()
 reexecWithOptionalContainer mprojectRoot =
     execWithOptionalContainer mprojectRoot getCmdArgs
   where
@@ -129,7 +117,7 @@ reexecWithOptionalContainer mprojectRoot =
               | configPlatform config == dockerContainerPlatform -> do
                   exePath <- liftIO getExecutablePath
                   cmdArgs args exePath
-              | otherwise -> throwM UnsupportedStackExeHostPlatformException
+              | otherwise -> throwIO UnsupportedStackExeHostPlatformException
             Just DockerStackExeImage -> do
                 progName <- liftIO getProgName
                 return (FP.takeBaseName progName, args, [], [])
@@ -197,20 +185,20 @@ reexecWithOptionalContainer mprojectRoot =
 --
 -- This takes an optional release action just like `reexecWithOptionalContainer`.
 execWithOptionalContainer
-    :: (StackM env m, HasConfig env)
+    :: HasConfig env
     => Maybe (Path Abs Dir)
-    -> GetCmdArgs env m
-    -> Maybe (m ())
+    -> GetCmdArgs env
+    -> Maybe (RIO env ())
     -> IO ()
-    -> Maybe (m ())
-    -> Maybe (m ())
-    -> m ()
+    -> Maybe (RIO env ())
+    -> Maybe (RIO env ())
+    -> RIO env ()
 execWithOptionalContainer mprojectRoot getCmdArgs mbefore inner mafter mrelease =
   do config <- view configL
      inContainer <- getInContainer
      isReExec <- view reExecL
      if | inContainer && not isReExec && (isJust mbefore || isJust mafter) ->
-            throwM OnlyOnHostException
+            throwIO OnlyOnHostException
         | inContainer ->
             liftIO (do inner
                        exitSuccess)
@@ -231,20 +219,21 @@ execWithOptionalContainer mprojectRoot getCmdArgs mbefore inner mafter mrelease
     fromMaybeAction (Just hook) = hook
 
 -- | Error if running in a container.
-preventInContainer :: (MonadIO m,MonadThrow m) => m () -> m ()
+preventInContainer :: MonadIO m => m () -> m ()
 preventInContainer inner =
   do inContainer <- getInContainer
      if inContainer
-        then throwM OnlyOnHostException
+        then throwIO OnlyOnHostException
         else inner
 
 -- | Run a command in a new Docker container, then exit the process.
-runContainerAndExit :: (StackM env m, HasConfig env)
-  => GetCmdArgs env m
+runContainerAndExit
+  :: HasConfig env
+  => GetCmdArgs env
   -> Maybe (Path Abs Dir) -- ^ Project root (maybe)
-  -> m ()              -- ^ Action to run before
-  -> m ()              -- ^ Action to run after
-  -> m ()
+  -> RIO env ()  -- ^ Action to run before
+  -> RIO env ()  -- ^ Action to run after
+  -> RIO env ()
 runContainerAndExit getCmdArgs
                     mprojectRoot
                     before
@@ -270,7 +259,7 @@ runContainerAndExit getCmdArgs
          image = dockerImage docker
      when (isRemoteDocker &&
            maybe False (isInfixOf "boot2docker") dockerCertPath)
-          ($logWarn "Warning: Using boot2docker is NOT supported, and not likely to perform well.")
+          (logWarn "Warning: Using boot2docker is NOT supported, and not likely to perform well.")
      maybeImageInfo <- inspect envOverride image
      imageInfo@Inspect{..} <- case maybeImageInfo of
        Just ii -> return ii
@@ -364,9 +353,9 @@ runContainerAndExit getCmdArgs
          ,args])
      before
 #ifndef WINDOWS
-     runInBase <- Control.liftBaseWith $ \run -> return (void . run)
+     run <- askRunInIO
      oldHandlers <- forM [sigINT,sigABRT,sigHUP,sigPIPE,sigTERM,sigUSR1,sigUSR2] $ \sig -> do
-       let sigHandler = runInBase $ do
+       let sigHandler = run $ do
              readProcessNull Nothing envOverride "docker"
                              ["kill","--signal=" ++ show sig,containerID]
              when (sig `elem` [sigTERM,sigABRT]) $ do
@@ -414,8 +403,7 @@ runContainerAndExit getCmdArgs
     sshRelDir = $(mkRelDir ".ssh/")
 
 -- | Clean-up old docker images and containers.
-cleanup :: (StackM env m, HasConfig env)
-        => CleanupOpts -> m ()
+cleanup :: HasConfig env => CleanupOpts -> RIO env ()
 cleanup opts =
   do config <- view configL
      let docker = configDocker config
@@ -474,16 +462,16 @@ cleanup opts =
         [] -> return ()
         (c:_):t:v:_ ->
           do args <- if | toUpper c == 'R' && t == imageStr ->
-                            do $logInfo (concatT ["Removing image: '",v,"'"])
+                            do logInfo (concatT ["Removing image: '",v,"'"])
                                return ["rmi",v]
                         | toUpper c == 'R' && t == containerStr ->
-                            do $logInfo (concatT ["Removing container: '",v,"'"])
+                            do logInfo (concatT ["Removing container: '",v,"'"])
                                return ["rm","-f",v]
                         | otherwise -> throwM (InvalidCleanupCommandException line)
              e <- try (readDockerProcess envOverride Nothing args)
              case e of
                Left ex@ProcessFailed{} ->
-                 $logError (concatT ["Could not remove: '",v,"': ", show ex])
+                 logError (concatT ["Could not remove: '",v,"': ", show ex])
                Left e' -> throwM e'
                Right _ -> return ()
         _ -> throwM (InvalidCleanupCommandException line)
@@ -495,12 +483,12 @@ cleanup opts =
                   | repo == "<none>" -> (hash,[])
                   | tag == "<none>" -> (hash,[repo])
                   | otherwise -> (hash,[repo ++ ":" ++ tag])
-                _ -> throw (InvalidImagesOutputException line)
+                _ -> impureThrow (InvalidImagesOutputException line)
     parseContainersOut = map parseContainer . drop 1 . lines . decodeUtf8
       where parseContainer line =
               case words line of
-                hash:image:rest -> (hash,(image,last rest))
-                _ -> throw (InvalidPSOutputException line)
+                hash:image:rest | last:_ <- reverse rest -> (hash,(image,last))
+                _ -> impureThrow (InvalidPSOutputException line)
     buildPlan curTime
               imagesLastUsed
               imageRepos
@@ -641,17 +629,17 @@ cleanup opts =
     containerStr = "container"
 
 -- | Inspect Docker image or container.
-inspect :: (MonadIO m,MonadLogger m,MonadBaseControl IO m,MonadCatch m)
+inspect :: (MonadUnliftIO m,MonadLogger m)
         => EnvOverride -> String -> m (Maybe Inspect)
 inspect envOverride image =
   do results <- inspects envOverride [image]
      case Map.toList results of
        [] -> return Nothing
        [(_,i)] -> return (Just i)
-       _ -> throwM (InvalidInspectOutputException "expect a single result")
+       _ -> throwIO (InvalidInspectOutputException "expect a single result")
 
 -- | Inspect multiple Docker images and/or containers.
-inspects :: (MonadIO m, MonadLogger m, MonadBaseControl IO m, MonadCatch m)
+inspects :: (MonadUnliftIO m, MonadLogger m)
          => EnvOverride -> [String] -> m (Map String Inspect)
 inspects _ [] = return Map.empty
 inspects envOverride images =
@@ -661,14 +649,15 @@ inspects envOverride images =
        Right inspectOut ->
          -- filtering with 'isAscii' to workaround @docker inspect@ output containing invalid UTF-8
          case eitherDecode (LBS.pack (filter isAscii (decodeUtf8 inspectOut))) of
-           Left msg -> throwM (InvalidInspectOutputException msg)
+           Left msg -> throwIO (InvalidInspectOutputException msg)
            Right results -> return (Map.fromList (map (\r -> (iiId r,r)) results))
        Left (ProcessFailed _ _ _ err)
-         | "Error: No such image" `LBS.isPrefixOf` err -> return Map.empty
-       Left e -> throwM e
+         |  any (`LBS.isPrefixOf` err) missingImagePrefixes -> return Map.empty
+       Left e -> throwIO e
+  where missingImagePrefixes = ["Error: No such image", "Error: No such object:"]
 
 -- | Pull latest version of configured Docker image from registry.
-pull :: (StackM env m, HasConfig env) => m ()
+pull :: HasConfig env => RIO env ()
 pull =
   do config <- view configL
      let docker = configDocker config
@@ -680,9 +669,9 @@ pull =
 pullImage :: (MonadLogger m,MonadIO m,MonadThrow m)
           => EnvOverride -> DockerOpts -> String -> m ()
 pullImage envOverride docker image =
-  do $logInfo (concatT ["Pulling image from registry: '",image,"'"])
+  do logInfo (concatT ["Pulling image from registry: '",image,"'"])
      when (dockerRegistryLogin docker)
-          (do $logInfo "You may need to log in."
+          (do logInfo "You may need to log in."
               callProcess $ Cmd
                 Nothing
                 "docker"
@@ -706,30 +695,30 @@ pullImage envOverride docker image =
      ec <- liftIO (waitForProcess ph)
      case ec of
        ExitSuccess -> return ()
-       ExitFailure _ -> throwM (PullFailedException image)
+       ExitFailure _ -> throwIO (PullFailedException image)
 
 -- | Check docker version (throws exception if incorrect)
 checkDockerVersion
-    :: (MonadIO m, MonadLogger m, MonadBaseControl IO m, MonadCatch m)
+    :: (MonadUnliftIO m, MonadLogger m)
     => EnvOverride -> DockerOpts -> m ()
 checkDockerVersion envOverride docker =
   do dockerExists <- doesExecutableExist envOverride "docker"
-     unless dockerExists (throwM DockerNotInstalledException)
+     unless dockerExists (throwIO DockerNotInstalledException)
      dockerVersionOut <- readDockerProcess envOverride Nothing ["--version"]
      case words (decodeUtf8 dockerVersionOut) of
        (_:_:v:_) ->
          case parseVersionFromString (stripVersion v) of
            Just v'
              | v' < minimumDockerVersion ->
-               throwM (DockerTooOldException minimumDockerVersion v')
+               throwIO (DockerTooOldException minimumDockerVersion v')
              | v' `elem` prohibitedDockerVersions ->
-               throwM (DockerVersionProhibitedException prohibitedDockerVersions v')
+               throwIO (DockerVersionProhibitedException prohibitedDockerVersions v')
              | not (v' `withinRange` dockerRequireDockerVersion docker) ->
-               throwM (BadDockerVersionException (dockerRequireDockerVersion docker) v')
+               throwIO (BadDockerVersionException (dockerRequireDockerVersion docker) v')
              | otherwise ->
                return ()
-           _ -> throwM InvalidVersionOutputException
-       _ -> throwM InvalidVersionOutputException
+           _ -> throwIO InvalidVersionOutputException
+       _ -> throwIO InvalidVersionOutputException
   where minimumDockerVersion = $(mkVersion "1.6.0")
         prohibitedDockerVersions = []
         stripVersion v = takeWhile (/= '-') (dropWhileEnd (not . isDigit) v)
@@ -747,14 +736,14 @@ reset maybeProjectRoot keepHome = do
 
 -- | The Docker container "entrypoint": special actions performed when first entering
 -- a container, such as switching the UID/GID to the "outside-Docker" user's.
-entrypoint :: (MonadIO m, MonadBaseControl IO m, MonadCatch m, MonadLogger m)
+entrypoint :: (MonadUnliftIO m, MonadLogger m, MonadThrow m)
            => Config -> DockerEntrypoint -> m ()
 entrypoint config@Config{..} DockerEntrypoint{..} =
   modifyMVar_ entrypointMVar $ \alreadyRan -> do
     -- Only run the entrypoint once
     unless alreadyRan $ do
       envOverride <- getEnvOverride configPlatform
-      homeDir <- parseAbsDir =<< liftIO (getEnv "HOME")
+      homeDir <- liftIO $ parseAbsDir =<< getEnv "HOME"
       -- Get the UserEntry for the 'stack' user in the image, if it exists
       estackUserEntry0 <- liftIO $ tryJust (guard . isDoesNotExistError) $
         User.getUserEntryForName stackUserName
@@ -768,7 +757,7 @@ entrypoint config@Config{..} DockerEntrypoint{..} =
         Right ue -> do
           -- If the 'stack' user exists in the image, copy any build plans and package indices from
           -- its original home directory to the host's stack root, to avoid needing to download them
-          origStackHomeDir <- parseAbsDir (User.homeDirectory ue)
+          origStackHomeDir <- liftIO $ parseAbsDir (User.homeDirectory ue)
           let origStackRoot = origStackHomeDir </> $(mkRelDir ("." ++ stackProgName))
           buildPlanDirExists <- doesDirExist (buildPlanDir origStackRoot)
           when buildPlanDirExists $ do
@@ -863,9 +852,9 @@ removeDirectoryContents path excludeDirs excludeFiles =
 
 -- | Produce a strict 'S.ByteString' from the stdout of a
 -- process. Throws a 'ReadProcessException' exception if the
--- process fails.  Logs process's stderr using @$logError@.
+-- process fails.  Logs process's stderr using @logError@.
 readDockerProcess
-    :: (MonadIO m, MonadLogger m, MonadBaseControl IO m, MonadCatch m)
+    :: (MonadUnliftIO m, MonadLogger m)
     => EnvOverride -> Maybe (Path Abs Dir) -> [String] -> m BS.ByteString
 readDockerProcess envOverride mpwd = readProcessStdout mpwd envOverride "docker"
 
@@ -881,13 +870,13 @@ hostBinDir = "/opt/host/bin"
 decodeUtf8 :: BS.ByteString -> String
 decodeUtf8 bs = T.unpack (T.decodeUtf8 bs)
 
--- | Convenience function constructing message for @$log*@.
+-- | Convenience function constructing message for @log*@.
 concatT :: [String] -> Text
 concatT = T.pack . concat
 
 -- | Fail with friendly error if project root not set.
 fromMaybeProjectRoot :: Maybe (Path Abs Dir) -> Path Abs Dir
-fromMaybeProjectRoot = fromMaybe (throw CannotDetermineProjectRootException)
+fromMaybeProjectRoot = fromMaybe (impureThrow CannotDetermineProjectRootException)
 
 -- | Environment variable that contained the old sandbox ID.
 -- | Use of this variable is deprecated, and only used to detect old images.
@@ -942,10 +931,9 @@ instance FromJSON ImageConfig where
          <*> fmap join (o .:? "Entrypoint") .!= []
 
 -- | Function to get command and arguments to run in Docker container
-type GetCmdArgs env m
-   = (StackM env m, HasConfig env)
-  => DockerOpts
+type GetCmdArgs env
+   = DockerOpts
   -> EnvOverride
   -> Inspect
   -> Bool
-  -> m (FilePath,[String],[(String,String)],[Mount])
+  -> RIO env (FilePath,[String],[(String,String)],[Mount])
