@@ -1,3 +1,4 @@
+{-# LANGUAGE NoImplicitPrelude #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE TupleSections #-}
 module Stack.FileWatch
@@ -8,24 +9,16 @@ module Stack.FileWatch
 
 import Blaze.ByteString.Builder (toLazyByteString, copyByteString)
 import Blaze.ByteString.Builder.Char.Utf8 (fromShow)
-import Control.Concurrent.Async (race_)
-import Control.Concurrent.STM
-import Control.Exception (Exception, fromException, catch, throwIO)
-import Control.Exception.Safe (tryAny)
-import Control.Monad (forever, unless, when)
+import Control.Concurrent.STM (check)
+import Stack.Prelude
 import qualified Data.ByteString.Lazy as L
 import qualified Data.Map.Strict as Map
-import Data.Monoid ((<>))
-import Data.Set (Set)
 import qualified Data.Set as Set
-import Data.String (fromString)
-import Data.Traversable (forM)
 import GHC.IO.Exception
-import GHC.IO.Handle (hIsTerminalDevice)
 import Path
 import System.Console.ANSI
 import System.FSNotify
-import System.IO (Handle, stdout, stderr, hPutStrLn)
+import System.IO (stdout, stderr, hPutStrLn, getLine)
 
 -- | Print an exception to stderr
 printExceptionStderr :: Exception e => e -> IO ()
