@@ -1,4 +1,5 @@
--- FIXME See how much of this module can be deleted.
+{-# LANGUAGE NoImplicitPrelude #-}
+{-# LANGUAGE DataKinds #-}
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE BangPatterns #-}
 {-# LANGUAGE OverloadedStrings #-}
@@ -11,33 +12,23 @@
 
 module Stack.GhcPkg
   (getGlobalDB
-  ,EnvOverride
-  ,envHelper
   ,findGhcPkgField
   ,createDatabase
   ,unregisterGhcPkgId
   ,getCabalPkgVer
   ,ghcPkgExeName
+  ,ghcPkgPathEnvVar
   ,mkGhcPackagePath)
   where
 
-import           Control.Monad
-import           Control.Monad.Catch
-import           Control.Monad.IO.Class
-import           Control.Monad.Logger
-import           Control.Monad.Trans.Control
+import           Stack.Prelude
 import qualified Data.ByteString.Char8 as S8
-import           Data.Either
 import           Data.List
-import           Data.Maybe
-import           Data.Text (Text)
 import qualified Data.Text as T
 import qualified Data.Text.Encoding as T
-import           Data.Text.Extra (stripCR)
-import           Path (Path, Abs, Dir, toFilePath, parent, mkRelFile, (</>))
+import           Path (parent, mkRelFile, (</>))
 import           Path.Extra (toFilePathNoTrailingSep)
 import           Path.IO
-import           Prelude hiding (FilePath)
 import           Stack.Constants
 import           Stack.Types.Build
 import           Stack.Types.GhcPkgId
@@ -49,15 +40,15 @@ import           System.FilePath (searchPathSeparator)
 import           System.Process.Read
 
 -- | Get the global package database
-getGlobalDB :: (MonadIO m, MonadLogger m, MonadBaseControl IO m, MonadCatch m)
+getGlobalDB :: (MonadUnliftIO m, MonadLogger m)
             => EnvOverride -> WhichCompiler -> m (Path Abs Dir)
 getGlobalDB menv wc = do
-    $logDebug "Getting global package database location"
+    logDebug "Getting global package database location"
     -- This seems like a strange way to get the global package database
     -- location, but I don't know of a better one
-    bs <- ghcPkg menv wc [] ["list", "--global"] >>= either throwM return
+    bs <- ghcPkg menv wc [] ["list", "--global"] >>= either throwIO return
     let fp = S8.unpack $ stripTrailingColon $ firstLine bs
-    resolveDir' fp
+    liftIO $ resolveDir' fp
   where
     stripTrailingColon bs
         | S8.null bs = bs
@@ -66,7 +57,7 @@ getGlobalDB menv wc = do
     firstLine = S8.takeWhile (\c -> c /= '\r' && c /= '\n')
 
 -- | Run the ghc-pkg executable
-ghcPkg :: (MonadIO m, MonadLogger m, MonadBaseControl IO m, MonadCatch m)
+ghcPkg :: (MonadUnliftIO m, MonadLogger m)
        => EnvOverride
        -> WhichCompiler
        -> [Path Abs Dir]
@@ -84,7 +75,7 @@ ghcPkg menv wc pkgDbs args = do
     args' = packageDbFlags pkgDbs ++ args
 
 -- | Create a package database in the given directory, if it doesn't exist.
-createDatabase :: (MonadIO m, MonadLogger m, MonadBaseControl IO m, MonadCatch m)
+createDatabase :: (MonadUnliftIO m, MonadLogger m)
                => EnvOverride -> WhichCompiler -> Path Abs Dir -> m ()
 createDatabase menv wc db = do
     exists <- doesFileExist (db </> $(mkRelFile "package.cache"))
@@ -95,12 +86,12 @@ createDatabase menv wc db = do
         dirExists <- doesDirExist db
         args <- if dirExists
             then do
-                $logWarn $ T.pack $ concat
+                logWarn $ T.pack $ concat
                     [ "The package database located at "
                     , toFilePath db
                     , " is corrupted (missing its package.cache file)."
                     ]
-                $logWarn "Proceeding with a recache"
+                logWarn "Proceeding with a recache"
                 return ["--package-db", toFilePath db, "recache"]
             else do
                 -- Creating the parent doesn't seem necessary, as ghc-pkg
@@ -111,8 +102,8 @@ createDatabase menv wc db = do
         eres <- tryProcessStdout Nothing menv (ghcPkgExeName wc) args
         case eres of
             Left e -> do
-                $logError $ T.pack $ "Unable to create package database at " ++ toFilePath db
-                throwM e
+                logError $ T.pack $ "Unable to create package database at " ++ toFilePath db
+                throwIO e
             Right _ -> return ()
 
 -- | Get the name to use for "ghc-pkg", given the compiler version.
@@ -120,6 +111,11 @@ ghcPkgExeName :: WhichCompiler -> String
 ghcPkgExeName Ghc = "ghc-pkg"
 ghcPkgExeName Ghcjs = "ghcjs-pkg"
 
+-- | Get the environment variable to use for the package DB paths.
+ghcPkgPathEnvVar :: WhichCompiler -> Text
+ghcPkgPathEnvVar Ghc = "GHC_PACKAGE_PATH"
+ghcPkgPathEnvVar Ghcjs = "GHCJS_PACKAGE_PATH"
+
 -- | Get the necessary ghc-pkg flags for setting up the given package database
 packageDbFlags :: [Path Abs Dir] -> [String]
 packageDbFlags pkgDbs =
@@ -128,7 +124,7 @@ packageDbFlags pkgDbs =
 
 -- | Get the value of a field of the package.
 findGhcPkgField
-    :: (MonadIO m, MonadLogger m, MonadBaseControl IO m, MonadCatch m)
+    :: (MonadUnliftIO m, MonadLogger m)
     => EnvOverride
     -> WhichCompiler
     -> [Path Abs Dir] -- ^ package databases
@@ -149,7 +145,7 @@ findGhcPkgField menv wc pkgDbs name field = do
                 fmap (stripCR . T.decodeUtf8) $ listToMaybe $ S8.lines lbs
 
 -- | Get the version of the package
-findGhcPkgVersion :: (MonadIO m, MonadLogger m, MonadBaseControl IO m, MonadCatch m)
+findGhcPkgVersion :: (MonadUnliftIO m, MonadLogger m)
                   => EnvOverride
                   -> WhichCompiler
                   -> [Path Abs Dir] -- ^ package databases
@@ -161,10 +157,10 @@ findGhcPkgVersion menv wc pkgDbs name = do
         Just !v -> return (parseVersion v)
         _ -> return Nothing
 
-unregisterGhcPkgId :: (MonadIO m, MonadLogger m, MonadCatch m, MonadBaseControl IO m)
+unregisterGhcPkgId :: (MonadUnliftIO m, MonadLogger m)
                     => EnvOverride
                     -> WhichCompiler
-                    -> CompilerVersion
+                    -> CompilerVersion 'CVActual
                     -> Path Abs Dir -- ^ package database
                     -> GhcPkgId
                     -> PackageIdentifier
@@ -172,7 +168,7 @@ unregisterGhcPkgId :: (MonadIO m, MonadLogger m, MonadCatch m, MonadBaseControl
 unregisterGhcPkgId menv wc cv pkgDb gid ident = do
     eres <- ghcPkg menv wc [pkgDb] args
     case eres of
-        Left e -> $logWarn $ T.pack $ show e
+        Left e -> logWarn $ T.pack $ show e
         Right _ -> return ()
   where
     -- TODO ideally we'd tell ghc-pkg a GhcPkgId instead
@@ -183,16 +179,16 @@ unregisterGhcPkgId menv wc cv pkgDb gid ident = do
             _ -> ["--ipid", ghcPkgIdString gid])
 
 -- | Get the version of Cabal from the global package database.
-getCabalPkgVer :: (MonadThrow m, MonadIO m, MonadLogger m, MonadBaseControl IO m, MonadCatch m)
+getCabalPkgVer :: (MonadUnliftIO m, MonadLogger m)
                => EnvOverride -> WhichCompiler -> m Version
 getCabalPkgVer menv wc = do
-    $logDebug "Getting Cabal package version"
+    logDebug "Getting Cabal package version"
     mres <- findGhcPkgVersion
         menv
         wc
         [] -- global DB
         cabalPackageName
-    maybe (throwM $ Couldn'tFindPkgId cabalPackageName) return mres
+    maybe (throwIO $ Couldn'tFindPkgId cabalPackageName) return mres
 
 -- | Get the value for GHC_PACKAGE_PATH
 mkGhcPackagePath :: Bool -> Path Abs Dir -> Path Abs Dir -> [Path Abs Dir] -> Path Abs Dir -> Text
