@@ -1,3 +1,4 @@
+{-# LANGUAGE NoImplicitPrelude #-}
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE TemplateHaskell #-}
@@ -14,50 +15,27 @@ module Stack.Ghci
     , GhciPkgInfo(..)
     , GhciException(..)
     , ghci
-
-    -- TODO: Address what should and should not be exported.
-    , renderScriptGhci
-    , renderScriptIntero
     ) where
 
-import           Control.Applicative
-import           Control.Arrow (second)
-import           Control.Exception.Safe (tryAny)
-import           Control.Monad hiding (forM)
-import           Control.Monad.Catch
-import           Control.Monad.IO.Class
-import           Control.Monad.Logger
+import           Stack.Prelude
 import           Control.Monad.State.Strict (State, execState, get, modify)
-import           Control.Monad.Trans.Unlift (MonadBaseUnlift)
 import qualified Data.ByteString.Char8 as S8
-import           Data.Either
-import           Data.Function
 import           Data.List
 import           Data.List.Extra (nubOrd)
-import           Data.Map.Strict (Map)
 import qualified Data.Map.Strict as M
-import           Data.Maybe
-import           Data.Maybe.Extra (forMaybeM)
-import           Data.Monoid
-import           Data.Set (Set)
 import qualified Data.Set as S
-import           Data.String
-import           Data.Text (Text)
 import qualified Data.Text as T
-import           Data.Traversable (forM)
-import           Data.Typeable (Typeable)
 import qualified Distribution.PackageDescription as C
 import qualified Distribution.Text as C
 import           Path
 import           Path.Extra (toFilePathNoTrailingSep)
-import           Path.IO
-import           Prelude
+import           Path.IO hiding (withSystemTempDir)
 import           Stack.Build
 import           Stack.Build.Installed
 import           Stack.Build.Source
 import           Stack.Build.Target
 import           Stack.Config (getLocalPackages)
-import           Stack.Constants
+import           Stack.Constants.Config
 import           Stack.Exec
 import           Stack.Ghci.Script
 import           Stack.Package
@@ -69,8 +47,8 @@ import           Stack.Types.FlagName
 import           Stack.Types.Package
 import           Stack.Types.PackageIdentifier
 import           Stack.Types.PackageName
-import           Stack.Types.StackT
-import           Text.Read (readMaybe)
+import           Stack.Types.Runner
+import           System.IO (putStrLn, putStr, getLine)
 
 #ifndef WINDOWS
 import qualified System.Posix.Files as Posix
@@ -88,8 +66,9 @@ data GhciOpts = GhciOpts
     , ghciMainIs             :: !(Maybe Text)
     , ghciLoadLocalDeps      :: !Bool
     , ghciSkipIntermediate   :: !Bool
-    , ghciHidePackages       :: !Bool
+    , ghciHidePackages       :: !(Maybe Bool)
     , ghciNoBuild            :: !Bool
+    , ghciOnlyMain           :: !Bool
     } deriving Show
 
 -- | Necessary information to load a package or its components.
@@ -111,6 +90,7 @@ data GhciException
     | MissingFileTarget String
     | Can'tSpecifyFilesAndTargets
     | Can'tSpecifyFilesAndMainIs
+    | GhciTargetParseException [Text]
     deriving (Typeable)
 
 instance Exception GhciException
@@ -125,14 +105,17 @@ instance Show GhciException where
     show (MissingFileTarget name) =
         "Cannot find file target " ++ name
     show Can'tSpecifyFilesAndTargets =
-        "Cannot use 'stack ghci' with both file targets and build targets"
+        "Cannot use 'stack ghci' with both file targets and package targets"
     show Can'tSpecifyFilesAndMainIs =
         "Cannot use 'stack ghci' with both file targets and --main-is flag"
+    show (GhciTargetParseException xs) =
+        show (TargetParseException xs) ++
+        "\nNote that to specify options to be passed to GHCi, use the --ghci-options flag"
 
 -- | Launch a GHCi session for the given local package targets with the
 -- given options and configure it with the load paths and extensions
 -- of those targets.
-ghci :: (StackM r m, HasEnvConfig r, MonadBaseUnlift IO m) => GhciOpts -> m ()
+ghci :: HasEnvConfig env => GhciOpts -> RIO env ()
 ghci opts@GhciOpts{..} = do
     let buildOptsCLI = defaultBuildOptsCLI
             { boptsCLITargets = []
@@ -143,8 +126,9 @@ ghci opts@GhciOpts{..} = do
     -- Parse --main-is argument.
     mainIsTargets <- parseMainIsTargets buildOptsCLI ghciMainIs
     -- Parse to either file targets or build targets
-    etargets <- preprocessTargets ghciTargets
+    etargets <- preprocessTargets buildOptsCLI ghciTargets
     (inputTargets, mfileTargets) <- case etargets of
+        Right packageTargets -> return (packageTargets, Nothing)
         Left rawFileTargets -> do
             case mainIsTargets of
                 Nothing -> return ()
@@ -152,51 +136,58 @@ ghci opts@GhciOpts{..} = do
             -- Figure out targets based on filepath targets
             (targetMap, fileInfo, extraFiles) <- findFileTargets locals rawFileTargets
             return (targetMap, Just (fileInfo, extraFiles))
-        Right rawTargets -> do
-            (_,_,normalTargets) <- parseTargetsFromBuildOpts AllowNoTargets buildOptsCLI
-                { boptsCLITargets = rawTargets }
-            return (normalTargets, Nothing)
-    -- Make sure the targets are known.
-    checkTargets inputTargets
     -- Get a list of all the local target packages.
     localTargets <- getAllLocalTargets opts inputTargets mainIsTargets sourceMap
+    -- Get a list of all the non-local target packages.
+    nonLocalTargets <- getAllNonLocalTargets inputTargets
     -- Check if additional package arguments are sensible.
     addPkgs <- checkAdditionalPackages ghciAdditionalPackages
     -- Build required dependencies and setup local packages.
+    stackYaml <- view stackYamlL
     buildDepsAndInitialSteps opts (map (packageNameText . fst) localTargets)
+    targetWarnings stackYaml localTargets nonLocalTargets mfileTargets
     -- Load the list of modules _after_ building, to catch changes in unlisted dependencies (#1180)
     pkgs <- getGhciPkgInfos buildOptsCLI sourceMap addPkgs (fmap fst mfileTargets) localTargets
     checkForIssues pkgs
     -- Finally, do the invocation of ghci
-    runGhci opts localTargets mainIsTargets pkgs (maybe [] snd mfileTargets)
+    runGhci opts localTargets mainIsTargets pkgs (maybe [] snd mfileTargets) (nonLocalTargets ++ addPkgs)
 
-preprocessTargets :: (StackM r m) => [Text] -> m (Either [Path Abs File] [Text])
-preprocessTargets rawTargets = do
-    let (fileTargetsRaw, normalTargets) =
+preprocessTargets :: HasEnvConfig env => BuildOptsCLI -> [Text] -> RIO env (Either [Path Abs File] (Map PackageName Target))
+preprocessTargets buildOptsCLI rawTargets = do
+    let (fileTargetsRaw, normalTargetsRaw) =
             partition (\t -> ".hs" `T.isSuffixOf` t || ".lhs" `T.isSuffixOf` t)
                       rawTargets
-    fileTargets <- forM fileTargetsRaw $ \fp0 -> do
-        let fp = T.unpack fp0
-        mpath <- forgivingAbsence (resolveFile' fp)
-        case mpath of
-            Nothing -> throwM (MissingFileTarget fp)
-            Just path -> return path
-    case (null fileTargets, null normalTargets) of
-        (False, False) -> throwM Can'tSpecifyFilesAndTargets
-        (False, _) -> return (Left fileTargets)
-        _ -> return (Right normalTargets)
+    -- Only use file targets if we have no normal targets.
+    if not (null fileTargetsRaw) && null normalTargetsRaw
+        then do
+            fileTargets <- forM fileTargetsRaw $ \fp0 -> do
+                let fp = T.unpack fp0
+                mpath <- liftIO $ forgivingAbsence (resolveFile' fp)
+                case mpath of
+                    Nothing -> throwM (MissingFileTarget fp)
+                    Just path -> return path
+            return (Left fileTargets)
+        else do
+            -- Try parsing targets before checking if both file and
+            -- module targets are specified (see issue#3342).
+            (_,_,normalTargets) <- parseTargets AllowNoTargets buildOptsCLI { boptsCLITargets = normalTargetsRaw }
+                `catch` \ex -> case ex of
+                    TargetParseException xs -> throwM (GhciTargetParseException xs)
+                    _ -> throwM ex
+            unless (null fileTargetsRaw) $ throwM Can'tSpecifyFilesAndTargets
+            return (Right normalTargets)
 
-parseMainIsTargets :: (StackM r m, HasEnvConfig r) => BuildOptsCLI -> Maybe Text -> m (Maybe (Map PackageName SimpleTarget))
+parseMainIsTargets :: HasEnvConfig env => BuildOptsCLI -> Maybe Text -> RIO env (Maybe (Map PackageName Target))
 parseMainIsTargets buildOptsCLI mtarget = forM mtarget $ \target -> do
-     (_,_,targets) <- parseTargetsFromBuildOpts AllowNoTargets buildOptsCLI
+     (_,_,targets) <- parseTargets AllowNoTargets buildOptsCLI
          { boptsCLITargets = [target] }
      return targets
 
 findFileTargets
-    :: (StackM r m, HasEnvConfig r)
+    :: HasEnvConfig env
     => [LocalPackage]
     -> [Path Abs File]
-    -> m (Map PackageName SimpleTarget, Map PackageName (Set (Path Abs File)), [Path Abs File])
+    -> RIO env (Map PackageName Target, Map PackageName (Set (Path Abs File)), [Path Abs File])
 findFileTargets locals fileTargets = do
     filePackages <- forM locals $ \lp -> do
         (_,compFiles,_,_) <- getPackageFiles (packageFiles (lpPackage lp)) (lpCabalFile lp)
@@ -211,18 +202,20 @@ findFileTargets locals fileTargets = do
     results <- forM foundFileTargetComponents $ \(fp, xs) ->
         case xs of
             [] -> do
-                $prettyWarn $
-                    "Couldn't find a component for file target" <+>
-                    display fp <>
-                    ". Attempting to load anyway."
+                prettyWarn $ vsep
+                    [ "Couldn't find a component for file target" <+>
+                      display fp <>
+                      ". This means that the correct ghc options might not be used."
+                    , "Attempting to load the file anyway."
+                    ]
                 return $ Left fp
             [x] -> do
-                $prettyInfo $
+                prettyInfo $
                     "Using configuration for" <+> display x <+>
                     "to load" <+> display fp
                 return $ Right (fp, x)
             (x:_) -> do
-                $prettyWarn $
+                prettyWarn $
                     "Multiple components contain file target" <+>
                     display fp <> ":" <+>
                     mconcat (intersperse ", " (map display xs)) <> line <>
@@ -230,8 +223,8 @@ findFileTargets locals fileTargets = do
                 return $ Right (fp, x)
     let (extraFiles, associatedFiles) = partitionEithers results
         targetMap =
-            foldl unionSimpleTargets M.empty $
-            map (\(_, (name, comp)) -> M.singleton name (STLocalComps (S.singleton comp)))
+            foldl unionTargets M.empty $
+            map (\(_, (name, comp)) -> M.singleton name (TargetComps (S.singleton comp)))
                 associatedFiles
         infoMap =
             foldl (M.unionWith S.union) M.empty $
@@ -239,43 +232,28 @@ findFileTargets locals fileTargets = do
                 associatedFiles
     return (targetMap, infoMap, extraFiles)
 
-checkTargets
-    :: (StackM r m, HasEnvConfig r)
-    => Map PackageName SimpleTarget
-    -> m ()
-checkTargets mp = do
-    let filtered = M.filter (== STUnknown) mp
-    unless (M.null filtered) $ do
-        bconfig <- view buildConfigL
-        throwM $ UnknownTargets (M.keysSet filtered) M.empty (bcStackYaml bconfig)
-
 getAllLocalTargets
-    :: (StackM r m, HasEnvConfig r)
+    :: HasEnvConfig env
     => GhciOpts
-    -> Map PackageName SimpleTarget
-    -> Maybe (Map PackageName SimpleTarget)
+    -> Map PackageName Target
+    -> Maybe (Map PackageName Target)
     -> SourceMap
-    -> m [(PackageName, (Path Abs File, SimpleTarget))]
+    -> RIO env [(PackageName, (Path Abs File, Target))]
 getAllLocalTargets GhciOpts{..} targets0 mainIsTargets sourceMap = do
     -- Use the 'mainIsTargets' as normal targets, for CLI concision. See
     -- #1845. This is a little subtle - we need to do the target parsing
     -- independently in order to handle the case where no targets are
     -- specified.
-    let targets = maybe targets0 (unionSimpleTargets targets0) mainIsTargets
-    packages <- getLocalPackages
+    let targets = maybe targets0 (unionTargets targets0) mainIsTargets
+    packages <- lpProject <$> getLocalPackages
     -- Find all of the packages that are directly demanded by the
     -- targets.
     directlyWanted <-
         forMaybeM (M.toList packages) $
-        \(dir,treatLikeExtraDep) ->
-             do cabalfp <- findOrGenerateCabalFile dir
-                name <- parsePackageNameFromFilePath cabalfp
-                if treatLikeExtraDep
-                    then return Nothing
-                    else case M.lookup name targets of
-                             Just simpleTargets ->
-                                 return (Just (name, (cabalfp, simpleTargets)))
-                             Nothing -> return Nothing
+        \(name, lpv) ->
+                case M.lookup name targets of
+                  Just simpleTargets -> return (Just (name, (lpvCabalFP lpv, simpleTargets)))
+                  Nothing -> return Nothing
     -- Figure out
     let extraLoadDeps = getExtraLoadDeps ghciLoadLocalDeps sourceMap directlyWanted
     if (ghciSkipIntermediate && not ghciLoadLocalDeps) || null extraLoadDeps
@@ -283,12 +261,12 @@ getAllLocalTargets GhciOpts{..} targets0 mainIsTargets sourceMap = do
         else do
             let extraList = T.intercalate ", " (map (packageNameText . fst) extraLoadDeps)
             if ghciLoadLocalDeps
-                then $logInfo $ T.concat
+                then logInfo $ T.concat
                     [ "The following libraries will also be loaded into GHCi because "
                     , "they are local dependencies of your targets, and you specified --load-local-deps:\n    "
                     , extraList
                     ]
-                else $logInfo $ T.concat
+                else logInfo $ T.concat
                     [ "The following libraries will also be loaded into GHCi because "
                     , "they are intermediate dependencies of your targets:\n    "
                     , extraList
@@ -296,7 +274,15 @@ getAllLocalTargets GhciOpts{..} targets0 mainIsTargets sourceMap = do
                     ]
             return (directlyWanted ++ extraLoadDeps)
 
-buildDepsAndInitialSteps :: (StackM r m, HasEnvConfig r, MonadBaseUnlift IO m) => GhciOpts -> [Text] -> m ()
+getAllNonLocalTargets
+    :: Map PackageName Target
+    -> RIO env [PackageName]
+getAllNonLocalTargets targets = do
+  let isNonLocal (TargetAll Dependency) = True
+      isNonLocal _ = False
+  return $ map fst $ filter (isNonLocal . snd) (M.toList targets)
+
+buildDepsAndInitialSteps :: HasEnvConfig env => GhciOpts -> [Text] -> RIO env ()
 buildDepsAndInitialSteps GhciOpts{..} targets0 = do
     let targets = targets0 ++ map T.pack ghciAdditionalPackages
     -- If necessary, do the build, for local packagee targets, only do
@@ -311,8 +297,8 @@ buildDepsAndInitialSteps GhciOpts{..} targets0 = do
         case eres of
             Right () -> return ()
             Left err -> do
-                $prettyError $ fromString (show err)
-                $prettyWarn "Build failed, but optimistically launching GHCi anyway"
+                prettyError $ fromString (show err)
+                prettyWarn "Build failed, but trying to launch GHCi anyway"
 
 checkAdditionalPackages :: MonadThrow m => [String] -> m [PackageName]
 checkAdditionalPackages pkgs = forM pkgs $ \name -> do
@@ -321,39 +307,51 @@ checkAdditionalPackages pkgs = forM pkgs $ \name -> do
     maybe (throwM $ InvalidPackageOption name) return mres
 
 runGhci
-    :: (StackM r m, HasEnvConfig r)
+    :: HasEnvConfig env
     => GhciOpts
-    -> [(PackageName, (Path Abs File, SimpleTarget))]
-    -> Maybe (Map PackageName SimpleTarget)
+    -> [(PackageName, (Path Abs File, Target))]
+    -> Maybe (Map PackageName Target)
     -> [GhciPkgInfo]
     -> [Path Abs File]
-    -> m ()
-runGhci GhciOpts{..} targets mainIsTargets pkgs extraFiles = do
+    -> [PackageName]
+    -> RIO env ()
+runGhci GhciOpts{..} targets mainIsTargets pkgs extraFiles exposePackages = do
     config <- view configL
     wc <- view $ actualCompilerVersionL.whichCompilerL
-    let pkgopts = hidePkgOpt ++ genOpts ++ ghcOpts
-        hidePkgOpt = if null pkgs || not ghciHidePackages then [] else ["-hide-all-packages"]
+    let pkgopts = hidePkgOpts ++ genOpts ++ ghcOpts
+        shouldHidePackages =
+          fromMaybe (not (null pkgs && null exposePackages)) ghciHidePackages
+        hidePkgOpts =
+          if shouldHidePackages
+            then "-hide-all-packages" :
+              -- This is necessary, because current versions of ghci
+              -- will entirely fail to start if base isn't visible. This
+              -- is because it tries to use the interpreter to set
+              -- buffering options on standard IO.
+              "-package" : "base" :
+              concatMap (\n -> ["-package", packageNameString n]) exposePackages
+            else []
         oneWordOpts bio
-            | ghciHidePackages = bioOneWordOpts bio ++ bioPackageFlags bio
+            | shouldHidePackages = bioOneWordOpts bio ++ bioPackageFlags bio
             | otherwise = bioOneWordOpts bio
         genOpts = nubOrd (concatMap (concatMap (oneWordOpts . snd) . ghciPkgOpts) pkgs)
         (omittedOpts, ghcOpts) = partition badForGhci $
-            concatMap (concatMap (bioOpts . snd) . ghciPkgOpts) pkgs ++
-            getUserOptions Nothing ++
-            concatMap (getUserOptions . Just . ghciPkgName) pkgs
-        getUserOptions mpkg =
-            map T.unpack (M.findWithDefault [] mpkg (unGhcOptions (configGhcOptions config)))
+            concatMap (concatMap (bioOpts . snd) . ghciPkgOpts) pkgs ++ map T.unpack
+              ( fold (configGhcOptionsByCat config) -- include everything, locals, and targets
+             ++ concatMap (getUserOptions . ghciPkgName) pkgs
+              )
+        getUserOptions pkg = M.findWithDefault [] pkg (configGhcOptionsByName config)
         badForGhci x =
             isPrefixOf "-O" x || elem x (words "-debug -threaded -ticky -static -Werror")
     unless (null omittedOpts) $
-        $logWarn
+        logWarn
             ("The following GHC options are incompatible with GHCi and have not been passed to it: " <>
              T.unwords (map T.pack (nubOrd omittedOpts)))
     oiDir <- view objectInterfaceDirL
     let odir =
             [ "-odir=" <> toFilePathNoTrailingSep oiDir
             , "-hidir=" <> toFilePathNoTrailingSep oiDir ]
-    $logInfo
+    logInfo
         ("Configuring GHCi with the following packages: " <>
          T.intercalate ", " (map (packageNameText . ghciPkgName) pkgs))
     let execGhci extras = do
@@ -365,73 +363,66 @@ runGhci GhciOpts{..} targets mainIsTargets pkgs extraFiles = do
                  -- not include CWD. If there aren't any packages, CWD
                  -- is included.
                   (if null pkgs then id else ("-i" : )) $
-                  odir <> pkgopts <> map T.unpack ghciGhcOptions <> ghciArgs <> extras)
-        interrogateExeForRenderFunction = do
-            menv <- liftIO $ configEnvOverride config defaultEnvSettings
-            output <- execObserve menv (fromMaybe (compilerExeName wc) ghciGhcCommand) ["--version"]
-            if "Intero" `isPrefixOf` output
-                then return renderScriptIntero
-                else return renderScriptGhci
+                  odir <> pkgopts <> extras <> map T.unpack ghciGhcOptions <> ghciArgs)
+        -- TODO: Consider optimizing this check. Perhaps if no
+        -- "with-ghc" is specified, assume that it is not using intero.
+        checkIsIntero =
+            -- Optimization dependent on the behavior of renderScript -
+            -- it doesn't matter if it's intero or ghci when loading
+            -- multiple packages.
+            case pkgs of
+                [_] -> do
+                    menv <- liftIO $ configEnvOverride config defaultEnvSettings
+                    output <- execObserve menv (fromMaybe (compilerExeName wc) ghciGhcCommand) ["--version"]
+                    return $ "Intero" `isPrefixOf` output
+                _ -> return False
     withSystemTempDir "ghci" $ \tmpDirectory -> do
         macrosOptions <- writeMacrosFile tmpDirectory pkgs
         if ghciNoLoadModules
             then execGhci macrosOptions
             else do
                 checkForDuplicateModules pkgs
-                renderFn <- interrogateExeForRenderFunction
+                isIntero <- checkIsIntero
                 bopts <- view buildOptsL
                 mainFile <- figureOutMainFile bopts mainIsTargets targets pkgs
-                scriptPath <- writeGhciScript tmpDirectory (renderFn pkgs mainFile extraFiles)
+                scriptPath <- writeGhciScript tmpDirectory (renderScript isIntero pkgs mainFile ghciOnlyMain extraFiles)
                 execGhci (macrosOptions ++ ["-ghci-script=" <> toFilePath scriptPath])
 
 writeMacrosFile :: (MonadIO m) => Path Abs Dir -> [GhciPkgInfo] -> m [String]
 writeMacrosFile tmpDirectory packages = do
-  preprocessCabalMacros packages macrosFile
+    preprocessCabalMacros packages macrosFile
   where
     macrosFile = tmpDirectory </> $(mkRelFile "cabal_macros.h")
 
 writeGhciScript :: (MonadIO m) => Path Abs Dir -> GhciScript -> m (Path Abs File)
 writeGhciScript tmpDirectory script = do
-  liftIO $ scriptToFile scriptPath script
-  setScriptPerms scriptFilePath
-  return scriptPath
+    liftIO $ scriptToFile scriptPath script
+    setScriptPerms scriptFilePath
+    return scriptPath
   where
     scriptPath = tmpDirectory </> $(mkRelFile "ghci-script")
     scriptFilePath = toFilePath scriptPath
 
-findOwningPackageForMain :: [GhciPkgInfo] -> Path Abs File -> Maybe GhciPkgInfo
-findOwningPackageForMain pkgs mainFile =
-  find (\pkg -> toFilePath (ghciPkgDir pkg) `isPrefixOf` toFilePath mainFile) pkgs
-
-renderScriptGhci :: [GhciPkgInfo] -> Maybe (Path Abs File) -> [Path Abs File] -> GhciScript
-renderScriptGhci pkgs mainFile extraFiles =
-  let addPhase    = mconcat $ fmap renderPkg pkgs
-      mainPhase   = case mainFile of
-                      Just path -> cmdAddFile path
-                      Nothing   -> mempty
-      modulePhase = cmdModule $ foldl' S.union S.empty (fmap ghciPkgModules pkgs)
-   in case getFileTargets pkgs <> extraFiles of
-          [] -> addPhase <> mainPhase <> modulePhase
-          fileTargets -> mconcat $ map cmdAddFile fileTargets
-  where
-    renderPkg pkg = cmdAdd (ghciPkgModules pkg)
-
-renderScriptIntero :: [GhciPkgInfo] -> Maybe (Path Abs File) -> [Path Abs File] -> GhciScript
-renderScriptIntero pkgs mainFile extraFiles =
-  let addPhase    = mconcat $ fmap renderPkg pkgs
-      mainPhase   = case mainFile of
-                      Just path ->
-                        case findOwningPackageForMain pkgs path of
-                          Just mainPkg -> cmdCdGhc (ghciPkgDir mainPkg) <> cmdAddFile path
-                          Nothing      -> cmdAddFile path
-                      Nothing   -> mempty
-      modulePhase = cmdModule $ foldl' S.union S.empty (fmap ghciPkgModules pkgs)
-   in case getFileTargets pkgs <> extraFiles of
-          [] -> addPhase <> mainPhase <> modulePhase
-          fileTargets -> mconcat $ map cmdAddFile fileTargets
-  where
-    renderPkg pkg = cmdCdGhc (ghciPkgDir pkg)
-                 <> cmdAdd (ghciPkgModules pkg)
+renderScript :: Bool -> [GhciPkgInfo] -> Maybe (Path Abs File) -> Bool -> [Path Abs File] -> GhciScript
+renderScript isIntero pkgs mainFile onlyMain extraFiles = do
+    let cdPhase = case (isIntero, pkgs) of
+          -- If only loading one package, set the cwd properly.
+          -- Otherwise don't try. See
+          -- https://github.com/commercialhaskell/stack/issues/3309
+          (True, [pkg]) -> cmdCdGhc (ghciPkgDir pkg)
+          _ -> mempty
+        addPhase = cmdAdd $ S.fromList (map Left allModules ++ addMain)
+        addMain = case mainFile of
+            Just path -> [Right path]
+            _ -> []
+        modulePhase = cmdModule $ S.fromList allModules
+        allModules = concatMap (S.toList . ghciPkgModules) pkgs
+    case getFileTargets pkgs <> extraFiles of
+        [] ->
+          if onlyMain
+            then cdPhase <> if isJust mainFile then cmdAdd (S.fromList addMain) else mempty
+            else cdPhase <> addPhase <> modulePhase
+        fileTargets -> cmdAdd (S.fromList (map Right fileTargets))
 
 -- Hacky check if module / main phase should be omitted. This should be
 -- improved if / when we have a better per-component load.
@@ -442,30 +433,30 @@ getFileTargets = concatMap (concatMap S.toList . maybeToList . ghciPkgTargetFile
 -- is none, sometimes it's unambiguous, sometimes it's
 -- ambiguous. Warns and returns nothing if it's ambiguous.
 figureOutMainFile
-    :: (StackM r m)
+    :: HasRunner env
     => BuildOpts
-    -> Maybe (Map PackageName SimpleTarget)
-    -> [(PackageName, (Path Abs File, SimpleTarget))]
+    -> Maybe (Map PackageName Target)
+    -> [(PackageName, (Path Abs File, Target))]
     -> [GhciPkgInfo]
-    -> m (Maybe (Path Abs File))
+    -> RIO env (Maybe (Path Abs File))
 figureOutMainFile bopts mainIsTargets targets0 packages = do
     case candidates of
         [] -> return Nothing
-        [c@(_,_,fp)] -> do $logInfo ("Using main module: " <> renderCandidate c)
+        [c@(_,_,fp)] -> do logInfo ("Using main module: " <> renderCandidate c)
                            return (Just fp)
         candidate:_ -> do
           borderedWarning $ do
-            $logWarn "The main module to load is ambiguous. Candidates are: "
-            forM_ (map renderCandidate candidates) $logWarn
-            $logWarn
+            logWarn "The main module to load is ambiguous. Candidates are: "
+            forM_ (map renderCandidate candidates) logWarn
+            logWarn
                 "You can specify which one to pick by: "
-            $logWarn
+            logWarn
                 (" * Specifying targets to stack ghci e.g. stack ghci " <>
                  sampleTargetArg candidate)
-            $logWarn
+            logWarn
                 (" * Specifying what the main is e.g. stack ghci " <>
                  sampleMainIsArg candidate)
-            $logWarn
+            logWarn
                 (" * Choosing from the candidate above [1.." <>
                 T.pack (show $ length candidates) <> "]")
           liftIO userOption
@@ -527,13 +518,13 @@ figureOutMainFile bopts mainIsTargets targets0 packages = do
         "--main-is " <> packageNameText pkg <> ":" <> renderComp comp
 
 getGhciPkgInfos
-    :: (StackM r m, HasEnvConfig r)
+    :: HasEnvConfig env
     => BuildOptsCLI
     -> SourceMap
     -> [PackageName]
     -> Maybe (Map PackageName (Set (Path Abs File)))
-    -> [(PackageName, (Path Abs File, SimpleTarget))]
-    -> m [GhciPkgInfo]
+    -> [(PackageName, (Path Abs File, Target))]
+    -> RIO env [GhciPkgInfo]
 getGhciPkgInfos buildOptsCLI sourceMap addPkgs mfileTargets localTargets = do
     menv <- getMinimalEnvOverride
     (installedMap, _, _, _) <- getInstalled
@@ -550,7 +541,7 @@ getGhciPkgInfos buildOptsCLI sourceMap addPkgs mfileTargets localTargets = do
 
 -- | Make information necessary to load the given package in GHCi.
 makeGhciPkgInfo
-    :: (StackM r m, HasEnvConfig r)
+    :: HasEnvConfig env
     => BuildOptsCLI
     -> SourceMap
     -> InstalledMap
@@ -559,8 +550,8 @@ makeGhciPkgInfo
     -> Maybe (Map PackageName (Set (Path Abs File)))
     -> PackageName
     -> Path Abs File
-    -> SimpleTarget
-    -> m GhciPkgInfo
+    -> Target
+    -> RIO env GhciPkgInfo
 makeGhciPkgInfo buildOptsCLI sourceMap installedMap locals addPkgs mfileTargets name cabalfp target = do
     bopts <- view buildOptsL
     econfig <- view envConfigL
@@ -575,7 +566,11 @@ makeGhciPkgInfo buildOptsCLI sourceMap installedMap locals addPkgs mfileTargets
             , packageConfigCompilerVersion = compilerVersion
             , packageConfigPlatform = view platformL econfig
             }
-    (warnings,gpkgdesc) <- readPackageUnresolved cabalfp
+    -- TODO we've already parsed this information, otherwise we
+    -- wouldn't have figured out the cabalfp already. In the future:
+    -- retain that GenericPackageDescription in the relevant data
+    -- structures to avoid reparsing.
+    (gpkgdesc, _cabalfp) <- readPackageUnresolvedDir (parent cabalfp) True
 
     -- Source the package's *.buildinfo file created by configure if any. See
     -- https://www.haskell.org/cabal/users-guide/developing-packages.html#system-dependent-parameters
@@ -585,12 +580,18 @@ makeGhciPkgInfo buildOptsCLI sourceMap installedMap locals addPkgs mfileTargets
           | hasDotBuildinfo = Just (parent cabalfp </> buildinfofp)
           | otherwise = Nothing
     mbuildinfo <- forM mbuildinfofp readDotBuildinfo
-    let pkg =
+    let pdp = resolvePackageDescription config gpkgdesc
+        pkg =
             packageFromPackageDescription config (C.genPackageFlags gpkgdesc) $
-            maybe id C.updatePackageDescription mbuildinfo $
-            resolvePackageDescription config gpkgdesc
+            maybe
+              pdp
+              (\bi ->
+               let PackageDescriptionPair x y = pdp
+                in PackageDescriptionPair
+                    (C.updatePackageDescription bi x)
+                    (C.updatePackageDescription bi y))
+              mbuildinfo
 
-    mapM_ (printCabalFileWarning cabalfp) warnings
     (mods,files,opts) <- getPackageOpts (packageOpts pkg) sourceMap installedMap locals addPkgs cabalfp
     let filteredOpts = filterWanted opts
         filterWanted = M.filterWithKey (\k _ -> k `S.member` allWanted)
@@ -612,10 +613,12 @@ makeGhciPkgInfo buildOptsCLI sourceMap installedMap locals addPkgs mfileTargets
 -- NOTE: this should make the same choices as the components code in
 -- 'loadLocalPackage'. Unfortunately for now we reiterate this logic
 -- (differently).
-wantedPackageComponents :: BuildOpts -> SimpleTarget -> Package -> Set NamedComponent
-wantedPackageComponents _ (STLocalComps cs) _ = cs
-wantedPackageComponents bopts STLocalAll pkg = S.fromList $
-    (if packageHasLibrary pkg then [CLib] else []) ++
+wantedPackageComponents :: BuildOpts -> Target -> Package -> Set NamedComponent
+wantedPackageComponents _ (TargetComps cs) _ = cs
+wantedPackageComponents bopts (TargetAll ProjectPackage) pkg = S.fromList $
+    (case packageLibraries pkg of
+       NoLibraries -> []
+       HasLibraries _names -> [CLib]) ++ -- FIXME. This ignores sub libraries and foreign libraries. Is that OK?
     map CExe (S.toList (packageExes pkg)) <>
     (if boptsTests bopts then map CTest (M.keys (packageTests pkg)) else []) <>
     (if boptsBenchmarks bopts then map CBench (S.toList (packageBenchmarks pkg)) else [])
@@ -624,15 +627,15 @@ wantedPackageComponents _ _ _ = S.empty
 checkForIssues :: (MonadThrow m, MonadLogger m) => [GhciPkgInfo] -> m ()
 checkForIssues pkgs = do
     unless (null issues) $ borderedWarning $ do
-        $logWarn "Warning: There are cabal settings for this project which may prevent GHCi from loading your code properly."
-        $logWarn "In some cases it can also load some projects which would otherwise fail to build."
-        $logWarn ""
-        mapM_ $logWarn $ intercalate [""] issues
-        $logWarn ""
-        $logWarn "To resolve, remove the flag(s) from the cabal file(s) and instead put them at the top of the haskell files."
-        $logWarn ""
-        $logWarn "It isn't yet possible to load multiple packages into GHCi in all cases - see"
-        $logWarn "https://ghc.haskell.org/trac/ghc/ticket/10827"
+        logWarn "Warning: There are cabal settings for this project which may prevent GHCi from loading your code properly."
+        logWarn "In some cases it can also load some projects which would otherwise fail to build."
+        logWarn ""
+        mapM_ logWarn $ intercalate [""] issues
+        logWarn ""
+        logWarn "To resolve, remove the flag(s) from the cabal file(s) and instead put them at the top of the haskell files."
+        logWarn ""
+        logWarn "It isn't yet possible to load multiple packages into GHCi in all cases - see"
+        logWarn "https://ghc.haskell.org/trac/ghc/ticket/10827"
   where
     issues = concat
         [ mixedFlag "-XNoImplicitPrelude"
@@ -686,20 +689,20 @@ checkForIssues pkgs = do
 
 borderedWarning :: MonadLogger m => m a -> m a
 borderedWarning f = do
-    $logWarn ""
-    $logWarn "* * * * * * * *"
+    logWarn ""
+    logWarn "* * * * * * * *"
     x <- f
-    $logWarn "* * * * * * * *"
-    $logWarn ""
+    logWarn "* * * * * * * *"
+    logWarn ""
     return x
 
 checkForDuplicateModules :: (MonadThrow m, MonadLogger m) => [GhciPkgInfo] -> m ()
 checkForDuplicateModules pkgs = do
     unless (null duplicates) $ do
         borderedWarning $ do
-            $logWarn "The following modules are present in multiple packages:"
+            logWarn "The following modules are present in multiple packages:"
             forM_ duplicates $ \(mn, pns) -> do
-                $logWarn (" * " <> T.pack mn <> " (in " <> T.intercalate ", " (map packageNameText pns) <> ")")
+                logWarn (" * " <> T.pack mn <> " (in " <> T.intercalate ", " (map packageNameText pns) <> ")")
         throwM LoadingDuplicateModules
   where
     duplicates, allModules :: [(String, [PackageName])]
@@ -708,6 +711,40 @@ checkForDuplicateModules pkgs = do
         M.toList $ M.fromListWith (++) $
         concatMap (\pkg -> map ((, [ghciPkgName pkg]) . C.display) (S.toList (ghciPkgModules pkg))) pkgs
 
+targetWarnings
+  :: HasRunner env
+  => Path Abs File
+  -> [(PackageName, (Path Abs File, Target))]
+  -> [PackageName]
+  -> Maybe (Map PackageName (Set (Path Abs File)), [Path Abs File])
+  -> RIO env ()
+targetWarnings stackYaml localTargets nonLocalTargets mfileTargets = do
+  unless (null nonLocalTargets) $
+    prettyWarnL
+      [ flow "Some targets"
+      , parens $ fillSep $ punctuate "," $ map (styleGood . display) nonLocalTargets
+      , flow "are not local packages, and so cannot be directly loaded."
+      , flow "In future versions of stack, this might be supported - see"
+      , styleUrl "https://github.com/commercialhaskell/stack/issues/1441"
+      , "."
+      , flow "It can still be useful to specify these, as they will be passed to ghci via -package flags."
+      ]
+  when (null localTargets && isNothing mfileTargets) $
+      prettyWarn $ vsep
+          [ flow "No local targets specified, so ghci will not use any options from your package.yaml / *.cabal files."
+          , ""
+          , flow "Potential ways to resolve this:"
+          , bulletedList
+              [ fillSep
+                  [ flow "If you want to use the package.yaml / *.cabal package in the current directory, use"
+                  , styleShell "stack init"
+                  , flow "to create a new stack.yaml."
+                  ]
+              , flow "Add to the 'packages' field of" <+> display stackYaml
+              ]
+          , ""
+          ]
+
 -- Adds in intermediate dependencies between ghci targets. Note that it
 -- will return a Lib component for these intermediate dependencies even
 -- if they don't have a library (but that's fine for the usage within
@@ -718,8 +755,8 @@ checkForDuplicateModules pkgs = do
 getExtraLoadDeps
     :: Bool
     -> SourceMap
-    -> [(PackageName, (Path Abs File, SimpleTarget))]
-    -> [(PackageName, (Path Abs File, SimpleTarget))]
+    -> [(PackageName, (Path Abs File, Target))]
+    -> [(PackageName, (Path Abs File, Target))]
 getExtraLoadDeps loadAllDeps sourceMap targets =
     M.toList $
     (\mp -> foldl' (flip M.delete) mp (map fst targets)) $
@@ -730,25 +767,25 @@ getExtraLoadDeps loadAllDeps sourceMap targets =
     getDeps :: PackageName -> [PackageName]
     getDeps name =
         case M.lookup name sourceMap of
-            Just (PSLocal lp) -> M.keys (packageDeps (lpPackage lp))
+            Just (PSFiles lp _) -> M.keys (packageDeps (lpPackage lp)) -- FIXME just Local?
             _ -> []
-    go :: PackageName -> State (Map PackageName (Maybe (Path Abs File, SimpleTarget))) Bool
+    go :: PackageName -> State (Map PackageName (Maybe (Path Abs File, Target))) Bool
     go name = do
         cache <- get
         case (M.lookup name cache, M.lookup name sourceMap) of
             (Just (Just _), _) -> return True
             (Just Nothing, _) | not loadAllDeps -> return False
-            (_, Just (PSLocal lp)) -> do
+            (_, Just (PSFiles lp _)) -> do
                 let deps = M.keys (packageDeps (lpPackage lp))
                 shouldLoad <- liftM or $ mapM go deps
                 if shouldLoad
                     then do
-                        modify (M.insert name (Just (lpCabalFile lp, STLocalComps (S.singleton CLib))))
+                        modify (M.insert name (Just (lpCabalFile lp, TargetComps (S.singleton CLib))))
                         return True
                     else do
                         modify (M.insert name Nothing)
                         return False
-            (_, Just PSUpstream{}) -> return loadAllDeps
+            (_, Just PSIndex{}) -> return loadAllDeps
             (_, _) -> return False
 
 preprocessCabalMacros :: MonadIO m => [GhciPkgInfo] -> Path Abs File -> m [String]
@@ -773,21 +810,20 @@ setScriptPerms fp = do
         ]
 #endif
 
-unionSimpleTargets :: Ord k => Map k SimpleTarget -> Map k SimpleTarget -> Map k SimpleTarget
-unionSimpleTargets = M.unionWith $ \l r ->
+unionTargets :: Ord k => Map k Target -> Map k Target -> Map k Target
+unionTargets = M.unionWith $ \l r ->
     case (l, r) of
-        (STUnknown, _) -> r
-        (STNonLocal, _) -> r
-        (STLocalComps sl, STLocalComps sr) -> STLocalComps (S.union sl sr)
-        (STLocalComps _, STLocalAll) -> STLocalAll
-        (STLocalComps _, _) -> l
-        (STLocalAll, _) -> STLocalAll
+        (TargetAll Dependency, _) -> r
+        (TargetComps sl, TargetComps sr) -> TargetComps (S.union sl sr)
+        (TargetComps _, TargetAll ProjectPackage) -> TargetAll ProjectPackage
+        (TargetComps _, _) -> l
+        (TargetAll ProjectPackage, _) -> TargetAll ProjectPackage
 
-hasLocalComp :: (NamedComponent -> Bool) -> SimpleTarget -> Bool
+hasLocalComp :: (NamedComponent -> Bool) -> Target -> Bool
 hasLocalComp p t =
     case t of
-        STLocalComps s -> any p (S.toList s)
-        STLocalAll -> True
+        TargetComps s -> any p (S.toList s)
+        TargetAll ProjectPackage -> True
         _ -> False
 
 
@@ -816,7 +852,7 @@ getPackageOptsAndTargetFiles pwd pkg = do
           -- FIXME: use compilerOptionsCabalFlag
           map ("--ghc-option=" ++) (concatMap (ghcOptions . snd) (ghciPkgOpts pkg))
         , mapMaybe
-              (fmap toFilePath . stripDir pwd)
+              (fmap toFilePath . stripProperPrefix pwd)
               (S.toList (ghciPkgCFiles pkg) <> S.toList (ghciPkgModFiles pkg) <>
                [paths_foo | paths_foo_exists]))
 
