@@ -1,3 +1,4 @@
+{-# LANGUAGE NoImplicitPrelude #-}
 {-# LANGUAGE ConstraintKinds #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE DeriveDataTypeable #-}
@@ -18,27 +19,17 @@ module Stack.New
     , listTemplates)
     where
 
-import           Control.Monad
-import           Control.Monad.Catch
-import           Control.Monad.IO.Class
-import           Control.Monad.Logger
+import           Stack.Prelude
 import           Control.Monad.Trans.Writer.Strict
 import           Data.Aeson
 import           Data.Aeson.Types
 import qualified Data.ByteString as SB
 import qualified Data.ByteString.Lazy as LB
 import           Data.Conduit
-import           Data.Foldable (asum)
 import qualified Data.HashMap.Strict as HM
 import           Data.List
-import           Data.Map.Strict (Map)
 import qualified Data.Map.Strict as M
-import           Data.Maybe (fromMaybe)
-import           Data.Maybe.Extra (mapMaybeM)
-import           Data.Monoid
-import           Data.Set (Set)
 import qualified Data.Set as S
-import           Data.Text (Text)
 import qualified Data.Text as T
 import qualified Data.Text.Encoding as T
 import qualified Data.Text.Encoding.Error as T (lenientDecode)
@@ -46,17 +37,15 @@ import qualified Data.Text.IO as T
 import qualified Data.Text.Lazy as LT
 import           Data.Time.Calendar
 import           Data.Time.Clock
-import           Data.Typeable
 import qualified Data.Yaml as Yaml
 import           Network.HTTP.Download
 import           Network.HTTP.Simple
 import           Path
 import           Path.IO
-import           Prelude
 import           Stack.Constants
+import           Stack.Constants.Config
 import           Stack.Types.Config
 import           Stack.Types.PackageName
-import           Stack.Types.StackT
 import           Stack.Types.TemplateName
 import           System.Process.Run
 import           Text.Hastache
@@ -80,9 +69,7 @@ data NewOpts = NewOpts
     }
 
 -- | Create a new project with the given options.
-new
-    :: (StackM env m, HasConfig env)
-    => NewOpts -> Bool -> m (Path Abs Dir)
+new :: HasConfig env => NewOpts -> Bool -> RIO env (Path Abs Dir)
 new opts forceOverwrite = do
     when (newOptsProjectName opts `elem` wiredInPackages) $
       throwM $ Can'tUseWiredInName (newOptsProjectName opts)
@@ -119,7 +106,7 @@ new opts forceOverwrite = do
                           LocalTemp -> "Loading local"
                           RemoteTemp -> "Downloading"
          in
-        $logInfo
+        logInfo
             (loading <> " template \"" <> templateName template <>
              "\" to create project \"" <>
              packageNameText project <>
@@ -132,8 +119,10 @@ data TemplateFrom = LocalTemp | RemoteTemp
 
 -- | Download and read in a template's text content.
 loadTemplate
-    :: forall env m. (StackM env m, HasConfig env)
-    => TemplateName -> (TemplateFrom -> m ()) -> m Text
+    :: forall env. HasConfig env
+    => TemplateName
+    -> (TemplateFrom -> RIO env ())
+    -> RIO env Text
 loadTemplate name logIt = do
     templateDir <- view $ configL.to templatesDir
     case templatePath name of
@@ -154,9 +143,9 @@ loadTemplate name logIt = do
                         Nothing -> throwM e
                 )
   where
-    loadLocalFile :: Path b File -> m Text
+    loadLocalFile :: Path b File -> RIO env Text
     loadLocalFile path = do
-        $logDebug ("Opening local template: \"" <> T.pack (toFilePath path)
+        logDebug ("Opening local template: \"" <> T.pack (toFilePath path)
                                                 <> "\"")
         exists <- doesFileExist path
         if exists
@@ -164,7 +153,7 @@ loadTemplate name logIt = do
             else throwM (FailedToLoadTemplate name (toFilePath path))
     relRequest :: MonadThrow n => Path Rel File -> n Request
     relRequest rel = parseRequest (defaultTemplateUrl <> "/" <> toFilePath rel)
-    downloadTemplate :: Request -> Path Abs File -> m Text
+    downloadTemplate :: Request -> Path Abs File -> RIO env Text
     downloadTemplate req path = do
         logIt RemoteTemp
         _ <-
@@ -176,13 +165,13 @@ loadTemplate name logIt = do
 
 -- | Apply and unpack a template into a directory.
 applyTemplate
-    :: (StackM env m, HasConfig env)
+    :: HasConfig env
     => PackageName
     -> TemplateName
     -> Map Text Text
     -> Path Abs Dir
     -> Text
-    -> m (Map (Path Abs File) LB.ByteString)
+    -> RIO env  (Map (Path Abs File) LB.ByteString)
 applyTemplate project template nonceParams dir templateText = do
     config <- view configL
     currentYear <- do
@@ -205,7 +194,7 @@ applyTemplate project template nonceParams dir templateText = do
                  templateText
                  (mkStrContextM (contextFunction context)))
     unless (S.null missingKeys)
-         ($logInfo ("\n" <> T.pack (show (MissingParameters project template missingKeys (configUserConfigPath config))) <> "\n"))
+         (logInfo ("\n" <> T.pack (show (MissingParameters project template missingKeys (configUserConfigPath config))) <> "\n"))
     files :: Map FilePath LB.ByteString <-
         catch (execWriterT $
                yield (T.encodeUtf8 (LT.toStrict applied)) $$
@@ -262,8 +251,8 @@ writeTemplateFiles files =
 
 -- | Run any initialization functions, such as Git.
 runTemplateInits
-    :: (StackM env m, HasConfig env)
-    => Path Abs Dir -> m ()
+    :: HasConfig env
+    => Path Abs Dir -> RIO env ()
 runTemplateInits dir = do
     menv <- getMinimalEnvOverride
     config <- view configL
@@ -272,10 +261,10 @@ runTemplateInits dir = do
         Just Git ->
             catch (callProcess $ Cmd (Just dir) "git" menv ["init"])
                   (\(_ :: ProcessExitedUnsuccessfully) ->
-                         $logInfo "git init failed to run, ignoring ...")
+                         logInfo "git init failed to run, ignoring ...")
 
 -- | Display the set of templates accompanied with description if available.
-listTemplates :: StackM env m => m ()
+listTemplates :: HasLogFunc env => RIO env ()
 listTemplates = do
     templates <- getTemplates
     templateInfo <- getTemplateInfo
@@ -292,7 +281,7 @@ listTemplates = do
       else mapM_ (liftIO . T.putStrLn . templateName) (S.toList templates)
 
 -- | Get the set of templates.
-getTemplates :: StackM env m => m (Set TemplateName)
+getTemplates :: HasLogFunc env => RIO env (Set TemplateName)
 getTemplates = do
     req <- liftM setGithubHeaders (parseUrlThrow defaultTemplatesList)
     resp <- catch (httpJSON req) (throwM . FailedToDownloadTemplates)
@@ -300,13 +289,13 @@ getTemplates = do
         200 -> return $ unTemplateSet $ getResponseBody resp
         code -> throwM (BadTemplatesResponse code)
 
-getTemplateInfo :: StackM env m => m (Map Text TemplateInfo)
+getTemplateInfo :: HasLogFunc env => RIO env (Map Text TemplateInfo)
 getTemplateInfo = do
   req <- liftM setGithubHeaders (parseUrlThrow defaultTemplateInfoUrl)
   resp <- catch (liftM Right $ httpLbs req) (\(ex :: HttpException) -> return . Left $ "Failed to download template info. The HTTP error was: " <> show ex)
   case resp >>= is200 of
     Left err -> do
-      liftIO . putStrLn $ err
+      logInfo $ T.pack err
       return M.empty
     Right resp' ->
       case Yaml.decodeEither (LB.toStrict $ getResponseBody resp') :: Either String Object of
