@@ -1,8 +1,9 @@
+{-# LANGUAGE NoImplicitPrelude #-}
 {-# LANGUAGE ConstraintKinds #-}
+{-# LANGUAGE DataKinds #-}
 {-# LANGUAGE DeriveDataTypeable #-}
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE OverloadedStrings #-}
-{-# LANGUAGE TemplateHaskell #-}
 
 -- | Run commands in a nix-shell
 module Stack.Nix
@@ -11,21 +12,11 @@ module Stack.Nix
   ,nixHelpOptName
   ) where
 
-import           Control.Arrow ((***))
-import           Control.Exception (Exception,throw)
-import           Control.Monad hiding (mapM)
-import           Control.Monad.IO.Class (liftIO)
-import           Control.Monad.Logger (logDebug)
-import           Data.Maybe
-import           Data.Monoid
+import           Stack.Prelude
 import qualified Data.Text as T
-import           Data.Traversable
-import           Data.Typeable (Typeable)
 import           Data.Version (showVersion)
-import           Path
 import           Path.IO
 import qualified Paths_stack as Meta
-import           Prelude hiding (mapM) -- Fix redundant import warnings
 import           Stack.Config (getInNixShell, getInContainer)
 import           Stack.Config.Nix (nixCompiler)
 import           Stack.Constants (platformVariantEnvVar,inNixShellEnvVar,inContainerEnvVar)
@@ -33,9 +24,8 @@ import           Stack.Exec (exec)
 import           Stack.Types.Config
 import           Stack.Types.Docker
 import           Stack.Types.Nix
+import           Stack.Types.Runner
 import           Stack.Types.Compiler
-import           Stack.Types.Internal
-import           Stack.Types.StackT
 import           System.Environment (getArgs,getExecutablePath,lookupEnv)
 import qualified System.FilePath  as F
 import           System.Process.Read (getEnvOverride)
@@ -43,11 +33,11 @@ import           System.Process.Read (getEnvOverride)
 -- | If Nix is enabled, re-runs the currently running OS command in a Nix container.
 -- Otherwise, runs the inner action.
 reexecWithOptionalShell
-    :: (StackM env m, HasConfig env)
+    :: HasConfig env
     => Maybe (Path Abs Dir)
-    -> IO CompilerVersion
+    -> IO (CompilerVersion 'CVWanted)
     -> IO ()
-    -> m ()
+    -> RIO env ()
 reexecWithOptionalShell mprojectRoot getCompilerVersion inner =
   do config <- view configL
      inShell <- getInNixShell
@@ -67,11 +57,11 @@ reexecWithOptionalShell mprojectRoot getCompilerVersion inner =
 
 
 runShellAndExit
-    :: (StackM env m, HasConfig env)
+    :: HasConfig env
     => Maybe (Path Abs Dir)
-    -> IO CompilerVersion
-    -> m (String, [String])
-    -> m ()
+    -> IO (CompilerVersion 'CVWanted)
+    -> RIO env (String, [String])
+    -> RIO env ()
 runShellAndExit mprojectRoot getCompilerVersion getCmdArgs = do
      config <- view configL
      envOverride <- getEnvOverride (configPlatform config)
@@ -81,9 +71,9 @@ runShellAndExit mprojectRoot getCompilerVersion getCmdArgs = do
          nixInitFile (configNix config)
      compilerVersion <- liftIO getCompilerVersion
      inContainer <- getInContainer
+     ghc <- either throwIO return $ nixCompiler compilerVersion
      let pkgsInConfig = nixPackages (configNix config)
-         ghc = nixCompiler compilerVersion
-         pkgs = pkgsInConfig ++ [ghc, "git"]
+         pkgs = pkgsInConfig ++ [ghc, "git", "gcc"]
          pkgsStr = "[" <> T.intercalate " " pkgs <> "]"
          pureShell = nixPureShell (configNix config)
          addGCRoots = nixAddGCRoots (configNix config)
@@ -123,8 +113,8 @@ runShellAndExit mprojectRoot getCompilerVersion getCmdArgs = do
                            -- Using --run instead of --command so we cannot
                            -- end up in the nix-shell if stack build is Ctrl-C'd
      pathVar <- liftIO $ lookupEnv "PATH"
-     $logDebug $ "PATH is: " <> T.pack (show pathVar)
-     $logDebug $
+     logDebug $ "PATH is: " <> T.pack (show pathVar)
+     logDebug $
        "Using a nix-shell environment " <> (case mshellFile of
             Just path -> "from file: " <> T.pack (toFilePath path)
             Nothing -> "with nix packages: " <> T.intercalate ", " pkgs)
@@ -139,7 +129,7 @@ escape str = "'" ++ foldr (\c -> if c == '\'' then
 
 -- | Fail with friendly error if project root not set.
 fromMaybeProjectRoot :: Maybe (Path Abs Dir) -> Path Abs Dir
-fromMaybeProjectRoot = fromMaybe (throw CannotDetermineProjectRoot)
+fromMaybeProjectRoot = fromMaybe (impureThrow CannotDetermineProjectRoot)
 
 -- | Command-line argument for "nix"
 nixCmdName :: String
