@@ -1,9 +1,8 @@
+{-# LANGUAGE NoImplicitPrelude #-}
 {-# LANGUAGE ConstraintKinds       #-}
 {-# LANGUAGE FlexibleContexts      #-}
 {-# LANGUAGE OverloadedStrings     #-}
 {-# LANGUAGE RankNTypes            #-}
-{-# LANGUAGE TemplateHaskell       #-}
-{-# LANGUAGE ViewPatterns          #-}
 {-# LANGUAGE DeriveDataTypeable    #-}
 {-# LANGUAGE TypeFamilies          #-}
 -- Create a source distribution tarball
@@ -19,59 +18,52 @@ import qualified Codec.Archive.Tar.Entry as Tar
 import qualified Codec.Compression.GZip as GZip
 import           Control.Applicative
 import           Control.Concurrent.Execute (ActionContext(..))
-import           Control.Monad (unless, void, liftM, filterM, foldM, when)
-import           Control.Monad.Catch
-import           Control.Monad.IO.Class
-import           Control.Monad.Logger
+import           Stack.Prelude
 import           Control.Monad.Reader.Class (local)
-import           Control.Monad.Trans.Control (liftBaseWith)
-import           Control.Monad.Trans.Unlift (MonadBaseUnlift)
 import qualified Data.ByteString as S
 import qualified Data.ByteString.Char8 as S8
 import qualified Data.ByteString.Lazy as L
 import           Data.Char (toLower)
-import           Data.Data (Data, Typeable, cast, gmapT)
-import           Data.Either (partitionEithers)
-import           Data.IORef (newIORef, readIORef, writeIORef)
+import           Data.Data (cast)
 import           Data.List
 import           Data.List.Extra (nubOrd)
 import           Data.List.NonEmpty (NonEmpty)
 import qualified Data.List.NonEmpty as NE
 import qualified Data.Map.Strict as Map
-import           Data.Maybe (fromMaybe, catMaybes)
-import           Data.Monoid ((<>))
 import qualified Data.Set as Set
 import qualified Data.Text as T
 import qualified Data.Text.Encoding as T
+import qualified Data.Text.Encoding.Error as T
 import qualified Data.Text.Lazy as TL
 import qualified Data.Text.Lazy.Encoding as TLE
 import           Data.Time.Clock.POSIX
 import           Distribution.Package (Dependency (..))
 import qualified Distribution.PackageDescription as Cabal
 import qualified Distribution.PackageDescription.Check as Check
+import qualified Distribution.PackageDescription.Parse as Cabal
 import           Distribution.PackageDescription.PrettyPrint (showGenericPackageDescription)
-import           Distribution.Text (display)
-import           Distribution.Version (simplifyVersionRange, orLaterVersion, earlierVersion)
-import           Distribution.Version.Extra
+import qualified Distribution.Types.UnqualComponentName as Cabal
+import qualified Distribution.Text as Cabal
+import           Distribution.Version (simplifyVersionRange, orLaterVersion, earlierVersion, hasUpperBound, hasLowerBound)
 import           Lens.Micro (set)
 import           Path
-import           Path.IO hiding (getModificationTime, getPermissions)
-import           Prelude -- Fix redundant import warnings
+import           Path.IO hiding (getModificationTime, getPermissions, withSystemTempDir)
 import           Stack.Build (mkBaseConfigOpts, build)
 import           Stack.Build.Execute
 import           Stack.Build.Installed
-import           Stack.Build.Source (loadSourceMap, getDefaultPackageConfig)
-import           Stack.Build.Target
-import           Stack.Config (resolvePackageEntry, removePathFromPackageEntry)
+import           Stack.Build.Source (loadSourceMap)
+import           Stack.Build.Target hiding (PackageType (..))
+import           Stack.PackageLocation (resolveMultiPackageLocation)
+import           Stack.PrettyPrint
 import           Stack.Constants
 import           Stack.Package
 import           Stack.Types.Build
+import           Stack.Types.BuildPlan
 import           Stack.Types.Config
 import           Stack.Types.Package
 import           Stack.Types.PackageIdentifier
 import           Stack.Types.PackageName
-import           Stack.Types.StackT
-import           Stack.Types.StringError
+import           Stack.Types.Runner
 import           Stack.Types.Version
 import           System.Directory (getModificationTime, getPermissions)
 import qualified System.FilePath as FP
@@ -112,10 +104,10 @@ instance Show CheckException where
 -- tarball is not written to the disk and instead yielded as a lazy
 -- bytestring.
 getSDistTarball
-  :: (StackM env m, HasEnvConfig env)
+  :: HasEnvConfig env
   => Maybe PvpBounds            -- ^ Override Config value
   -> Path Abs Dir               -- ^ Path to local package
-  -> m (FilePath, L.ByteString, Maybe (PackageIdentifier, L.ByteString))
+  -> RIO env (FilePath, L.ByteString, Maybe (PackageIdentifier, L.ByteString))
   -- ^ Filename, tarball contents, and option cabal file revision to upload
 getSDistTarball mpvpBounds pkgDir = do
     config <- view configL
@@ -123,10 +115,10 @@ getSDistTarball mpvpBounds pkgDir = do
         tweakCabal = pvpBounds /= PvpBoundsNone
         pkgFp = toFilePath pkgDir
     lp <- readLocalPackage pkgDir
-    $logInfo $ "Getting file list for " <> T.pack pkgFp
+    logInfo $ "Getting file list for " <> T.pack pkgFp
     (fileList, cabalfp) <-  getSDistFileList lp
-    $logInfo $ "Building sdist tarball for " <> T.pack pkgFp
-    files <- normalizeTarballPaths (lines fileList)
+    logInfo $ "Building sdist tarball for " <> T.pack pkgFp
+    files <- normalizeTarballPaths (map (T.unpack . stripCR . T.pack) (lines fileList))
 
     -- We're going to loop below and eventually find the cabal
     -- file. When we do, we'll upload this reference, if the
@@ -151,13 +143,13 @@ getSDistTarball mpvpBounds pkgDir = do
             -- This is a cabal file, we're going to tweak it, but only
             -- tweak it as a revision.
             | tweakCabal && isCabalFp fp && asRevision = do
-                lbsIdent <- getCabalLbs pvpBounds (Just 1) $ toFilePath cabalfp
+                lbsIdent <- getCabalLbs pvpBounds (Just 1) cabalfp
                 liftIO (writeIORef cabalFileRevisionRef (Just lbsIdent))
                 packWith packFileEntry False fp
             -- Same, except we'll include the cabal file in the
             -- original tarball upload.
             | tweakCabal && isCabalFp fp = do
-                (_ident, lbs) <- getCabalLbs pvpBounds Nothing $ toFilePath cabalfp
+                (_ident, lbs) <- getCabalLbs pvpBounds Nothing cabalfp
                 currTime <- liftIO getPOSIXTime -- Seconds from UNIX epoch
                 tp <- liftIO $ tarPath False fp
                 return $ (Tar.fileEntry tp lbs) { Tar.entryTime = floor currTime }
@@ -171,14 +163,15 @@ getSDistTarball mpvpBounds pkgDir = do
     return (tarName, GZip.compress (Tar.write (dirEntries ++ fileEntries)), mcabalFileRevision)
 
 -- | Get the PVP bounds-enabled version of the given cabal file
-getCabalLbs :: (StackM env m, HasEnvConfig env)
+getCabalLbs :: HasEnvConfig env
             => PvpBoundsType
             -> Maybe Int -- ^ optional revision
-            -> FilePath
-            -> m (PackageIdentifier, L.ByteString)
-getCabalLbs pvpBounds mrev fp = do
-    path <- liftIO $ resolveFile' fp
-    (_warnings, gpd) <- readPackageUnresolved path
+            -> Path Abs File -- ^ cabal file
+            -> RIO env (PackageIdentifier, L.ByteString)
+getCabalLbs pvpBounds mrev cabalfp = do
+    (gpd, cabalfp') <- readPackageUnresolvedDir (parent cabalfp) False
+    unless (cabalfp == cabalfp')
+      $ error $ "getCabalLbs: cabalfp /= cabalfp': " ++ show (cabalfp, cabalfp')
     (_, sourceMap) <- loadSourceMap AllowNoTargets defaultBuildOptsCLI
     menv <- getMinimalEnvOverride
     (installedMap, _, _, _) <- getInstalled menv GetInstalledOpts
@@ -187,7 +180,10 @@ getCabalLbs pvpBounds mrev fp = do
                                 , getInstalledSymbols = False
                                 }
                                 sourceMap
-    let gpd' = gtraverseT (addBounds sourceMap installedMap) gpd
+    let internalPackages = Set.fromList $
+          gpdPackageName gpd :
+          map (fromCabalPackageName . Cabal.unqualComponentNameToPackageName . fst) (Cabal.condSubLibraries gpd)
+        gpd' = gtraverseT (addBounds internalPackages sourceMap installedMap) gpd
         gpd'' =
           case mrev of
             Nothing -> gpd'
@@ -201,25 +197,74 @@ getCabalLbs pvpBounds mrev fp = do
                   $ Cabal.packageDescription gpd'
                   }
               }
-    ident <- parsePackageIdentifierFromString $ display $ Cabal.package $ Cabal.packageDescription gpd''
+    ident <- parsePackageIdentifierFromString $ Cabal.display $ Cabal.package $ Cabal.packageDescription gpd''
+    -- Sanity rendering and reparsing the input, to ensure there are no
+    -- cabal bugs, since there have been bugs here before, and currently
+    -- are at the time of writing:
+    --
+    -- https://github.com/haskell/cabal/issues/1202
+    -- https://github.com/haskell/cabal/issues/2353
+    -- https://github.com/haskell/cabal/issues/4863 (current issue)
+    let roundtripErrs =
+          [ flow "Bug detected in Cabal library. ((parse . render . parse) === id) does not hold for the cabal file at"
+          <+> display cabalfp
+          , ""
+          ]
+    case Cabal.parseGenericPackageDescription (showGenericPackageDescription gpd) of
+      Cabal.ParseOk _ roundtripped
+        | roundtripped == gpd -> return ()
+        | otherwise -> do
+            prettyWarn $ vsep $ roundtripErrs ++
+              [ "This seems to be fixed in development versions of Cabal, but at time of writing, the fix is not in any released versions."
+              , ""
+              ,  "Please see this GitHub issue for status:" <+> styleUrl "https://github.com/commercialhaskell/stack/issues/3549"
+              , ""
+              , fillSep
+                [ flow "If the issue is closed as resolved, then you may be able to fix this by upgrading to a newer version of stack via"
+                , styleShell "stack upgrade"
+                , flow "for latest stable version or"
+                , styleShell "stack upgrade --git"
+                , flow "for the latest development version."
+                ]
+              , ""
+              , fillSep
+                [ flow "If the issue is fixed, but updating doesn't solve the problem, please check if there are similar open issues, and if not, report a new issue to the stack issue tracker, at"
+                , styleUrl "https://github.com/commercialhaskell/stack/issues/new"
+                ]
+              , ""
+              , flow "If the issue is not fixed, feel free to leave a comment on it indicating that you would like it to be fixed."
+              , ""
+              ]
+      Cabal.ParseFailed err -> do
+        prettyWarn $ vsep $ roundtripErrs ++
+          [ flow "In particular, parsing the rendered cabal file is yielding a parse error.  Please check if there are already issues tracking this, and if not, please report new issues to the stack and cabal issue trackers, via"
+          , bulletedList
+            [ styleUrl "https://github.com/commercialhaskell/stack/issues/new"
+            , styleUrl "https://github.com/haskell/cabal/issues/new"
+            ]
+          , flow $ "The parse error is: " ++ show err
+          , ""
+          ]
     return
       ( ident
       , TLE.encodeUtf8 $ TL.pack $ showGenericPackageDescription gpd''
       )
   where
-    addBounds :: SourceMap -> InstalledMap -> Dependency -> Dependency
-    addBounds sourceMap installedMap dep@(Dependency cname range) =
-      case lookupVersion (fromCabalPackageName cname) of
-        Nothing -> dep
-        Just version -> Dependency cname $ simplifyVersionRange
-          $ (if toAddUpper && not (hasUpper range) then addUpper version else id)
-          $ (if toAddLower && not (hasLower range) then addLower version else id)
-            range
+    addBounds :: Set PackageName -> SourceMap -> InstalledMap -> Dependency -> Dependency
+    addBounds internalPackages sourceMap installedMap dep@(Dependency cname range) =
+      if name `Set.member` internalPackages
+        then dep
+        else case foundVersion of
+          Nothing -> dep
+          Just version -> Dependency cname $ simplifyVersionRange
+            $ (if toAddUpper && not (hasUpperBound range) then addUpper version else id)
+            $ (if toAddLower && not (hasLowerBound range) then addLower version else id)
+              range
       where
-        lookupVersion name =
+        name = fromCabalPackageName cname
+        foundVersion =
           case Map.lookup name sourceMap of
-              Just (PSLocal lp) -> Just $ packageVersion $ lpPackage lp
-              Just (PSUpstream version _ _ _ _) -> Just version
+              Just ps -> Just (piiVersion ps)
               Nothing ->
                   case Map.lookup name installedMap of
                       Just (_, installed) -> Just (installedVersion installed)
@@ -247,12 +292,10 @@ gtraverseT f =
 -- | Read in a 'LocalPackage' config.  This makes some default decisions
 -- about 'LocalPackage' fields that might not be appropriate for other
 -- use-cases.
-readLocalPackage :: (StackM env m, HasEnvConfig env) => Path Abs Dir -> m LocalPackage
+readLocalPackage :: HasEnvConfig env => Path Abs Dir -> RIO env LocalPackage
 readLocalPackage pkgDir = do
-    cabalfp <- findOrGenerateCabalFile pkgDir
     config  <- getDefaultPackageConfig
-    (warnings,package) <- readPackage config cabalfp
-    mapM_ (printCabalFileWarning cabalfp) warnings
+    (package, cabalfp) <- readPackageDir config pkgDir True
     return LocalPackage
         { lpPackage = package
         , lpWanted = False -- HACK: makes it so that sdist output goes to a log instead of a file.
@@ -265,14 +308,15 @@ readLocalPackage pkgDir = do
         , lpTestBench = Nothing
         , lpForceDirty = False
         , lpDirtyFiles = Nothing
-        , lpNewBuildCache = Map.empty
-        , lpFiles = Set.empty
+        , lpNewBuildCaches = Map.empty
+        , lpComponentFiles = Map.empty
         , lpComponents = Set.empty
         , lpUnbuildable = Set.empty
+        , lpLocation = PLFilePath $ toFilePath pkgDir
         }
 
 -- | Returns a newline-separate list of paths, and the absolute path to the .cabal file.
-getSDistFileList :: (StackM env m, HasEnvConfig env) => LocalPackage -> m (String, Path Abs File)
+getSDistFileList :: HasEnvConfig env => LocalPackage -> RIO env (String, Path Abs File)
 getSDistFileList lp =
     withSystemTempDir (stackProgName <> "-sdist") $ \tmpdir -> do
         menv <- getMinimalEnvOverride
@@ -280,21 +324,21 @@ getSDistFileList lp =
         let boptsCli = defaultBuildOptsCLI
         baseConfigOpts <- mkBaseConfigOpts boptsCli
         (locals, _) <- loadSourceMap NeedTargets boptsCli
-        runInBase <- liftBaseWith $ \run -> return (void . run)
+        run <- askRunInIO
         withExecuteEnv menv bopts boptsCli baseConfigOpts locals
             [] [] [] -- provide empty list of globals. This is a hack around custom Setup.hs files
             $ \ee ->
-            withSingleContext runInBase ac ee task Nothing (Just "sdist") $ \_package cabalfp _pkgDir cabal _announce _console _mlogFile -> do
+            withSingleContext run ac ee task Nothing (Just "sdist") $ \_package cabalfp _pkgDir cabal _announce _console _mlogFile -> do
                 let outFile = toFilePath tmpdir FP.</> "source-files-list"
                 cabal KeepTHLoading ["sdist", "--list-sources", outFile]
-                contents <- liftIO (readFile outFile)
-                return (contents, cabalfp)
+                contents <- liftIO (S.readFile outFile)
+                return (T.unpack $ T.decodeUtf8With T.lenientDecode contents, cabalfp)
   where
     package = lpPackage lp
     ac = ActionContext Set.empty []
     task = Task
         { taskProvides = PackageIdentifier (packageName package) (packageVersion package)
-        , taskType = TTLocal lp
+        , taskType = TTFiles lp Local
         , taskConfigOpts = TaskConfigOpts
             { tcoMissing = Set.empty
             , tcoOpts = \_ -> ConfigureOpts [] []
@@ -302,14 +346,16 @@ getSDistFileList lp =
         , taskPresent = Map.empty
         , taskAllInOne = True
         , taskCachePkgSrc = CacheSrcLocal (toFilePath (lpDir lp))
+        , taskAnyMissing = True
+        , taskBuildTypeConfig = False
         }
 
-normalizeTarballPaths :: (StackM env m) => [FilePath] -> m [FilePath]
+normalizeTarballPaths :: HasRunner env => [FilePath] -> RIO env [FilePath]
 normalizeTarballPaths fps = do
     -- TODO: consider whether erroring out is better - otherwise the
     -- user might upload an incomplete tar?
     unless (null outsideDir) $
-        $logWarn $ T.concat
+        logWarn $ T.concat
             [ "Warning: These files are outside of the package directory, and will be omitted from the tarball: "
             , T.pack (show outsideDir)]
     return (nubOrd files)
@@ -337,10 +383,11 @@ dirsFromFiles dirs = Set.toAscList (Set.delete "." results)
 -- and will throw an exception in case of critical errors.
 --
 -- Note that we temporarily decompress the archive to analyze it.
-checkSDistTarball :: (StackM env m, HasEnvConfig env, MonadBaseUnlift IO m)
+checkSDistTarball
+  :: HasEnvConfig env
   => SDistOpts -- ^ The configuration of what to check
   -> Path Abs File -- ^ Absolute path to tarball
-  -> m ()
+  -> RIO env ()
 checkSDistTarball opts tarball = withTempTarGzContents tarball $ \pkgDir' -> do
     pkgDir  <- (pkgDir' </>) `liftM`
         (parseRelDir . FP.takeBaseName . FP.takeBaseName . toFilePath $ tarball)
@@ -348,17 +395,31 @@ checkSDistTarball opts tarball = withTempTarGzContents tarball $ \pkgDir' -> do
     when (sdoptsBuildTarball opts) (buildExtractedTarball pkgDir)
     unless (sdoptsIgnoreCheck opts) (checkPackageInExtractedTarball pkgDir)
 
-checkPackageInExtractedTarball :: (StackM env m, HasEnvConfig env, MonadBaseUnlift IO m)
+checkPackageInExtractedTarball
+  :: HasEnvConfig env
   => Path Abs Dir -- ^ Absolute path to tarball
-  -> m ()
+  -> RIO env ()
 checkPackageInExtractedTarball pkgDir = do
-    cabalfp <- findOrGenerateCabalFile pkgDir
-    name    <- parsePackageNameFromFilePath cabalfp
+    (gpd, _cabalfp) <- readPackageUnresolvedDir pkgDir True
+    let name = gpdPackageName gpd
     config  <- getDefaultPackageConfig
-    (gdesc, pkgDesc) <- readPackageDescriptionDir config pkgDir
-    $logInfo $
+    (gdesc, PackageDescriptionPair pkgDesc _) <- readPackageDescriptionDir config pkgDir False
+    logInfo $
         "Checking package '" <> packageNameText name <> "' for common mistakes"
-    let pkgChecks = Check.checkPackage gdesc (Just pkgDesc)
+    let pkgChecks =
+          -- MSS 2017-12-12: Try out a few different variants of
+          -- pkgDesc to try and provoke an error or warning. I don't
+          -- know why, but when using `Just pkgDesc`, it appears that
+          -- Cabal does not detect that `^>=` is used with
+          -- `cabal-version: 1.24` or earlier. It seems like pkgDesc
+          -- (the one we create) does not populate the `buildDepends`
+          -- field, whereas flattenPackageDescription from Cabal
+          -- does. In any event, using `Nothing` seems more logical
+          -- for this check anyway, and the fallback to `Just pkgDesc`
+          -- is just a crazy sanity check.
+          case Check.checkPackage gdesc Nothing of
+            [] -> Check.checkPackage gdesc (Just pkgDesc)
+            x -> x
     fileChecks <- liftIO $ Check.checkPackageFiles pkgDesc (toFilePath pkgDir)
     let checks = pkgChecks ++ fileChecks
         (errors, warnings) =
@@ -367,33 +428,25 @@ checkPackageInExtractedTarball pkgDir = do
               criticalIssue _ = False
           in partition criticalIssue checks
     unless (null warnings) $
-        $logWarn $ "Package check reported the following warnings:\n" <>
+        logWarn $ "Package check reported the following warnings:\n" <>
                    T.pack (intercalate "\n" . fmap show $ warnings)
     case NE.nonEmpty errors of
         Nothing -> return ()
         Just ne -> throwM $ CheckException ne
 
-buildExtractedTarball :: (StackM env m, HasEnvConfig env, MonadBaseUnlift IO m) => Path Abs Dir -> m ()
+buildExtractedTarball :: HasEnvConfig env => Path Abs Dir -> RIO env ()
 buildExtractedTarball pkgDir = do
   projectRoot <- view projectRootL
   envConfig <- view envConfigL
-  menv <- getMinimalEnvOverride
   localPackageToBuild <- readLocalPackage pkgDir
-  let packageEntries = bcPackageEntries (envConfigBuildConfig envConfig)
-      getPaths entry = do
-        resolvedEntry <- resolvePackageEntry menv projectRoot entry
-        return $ fmap fst resolvedEntry
-  allPackagePaths <- fmap mconcat (mapM getPaths packageEntries)
+  let packageEntries = bcPackages (envConfigBuildConfig envConfig)
+      getPaths = resolveMultiPackageLocation projectRoot
+  allPackagePaths <- fmap (map fst . mconcat) (mapM getPaths packageEntries)
   -- We remove the path based on the name of the package
   let isPathToRemove path = do
         localPackage <- readLocalPackage path
         return $ packageName (lpPackage localPackage) == packageName (lpPackage localPackageToBuild)
-  pathsToRemove <- filterM isPathToRemove allPackagePaths
-  let adjustPackageEntries entries path = do
-        adjustedPackageEntries <- mapM (removePathFromPackageEntry menv projectRoot path) entries
-        return (catMaybes adjustedPackageEntries)
-  entriesWithoutBuiltPackage <- foldM adjustPackageEntries packageEntries pathsToRemove
-  let newEntry = PackageEntry Nothing (PLFilePath (toFilePath pkgDir)) []
+  pathsToKeep <- filterM (fmap not . isPathToRemove) allPackagePaths
   newPackagesRef <- liftIO (newIORef Nothing)
   let adjustEnvForBuild env =
         let updatedEnvConfig = envConfig
@@ -402,7 +455,7 @@ buildExtractedTarball pkgDir = do
               }
         in set envConfigL updatedEnvConfig env
       updatePackageInBuildConfig buildConfig = buildConfig
-        { bcPackageEntries = newEntry : entriesWithoutBuiltPackage
+        { bcPackages = map (PLFilePath . toFilePath) $ pkgDir : pathsToKeep
         , bcConfig = (bcConfig buildConfig)
                      { configBuild = defaultBuildOpts
                        { boptsTests = True
@@ -414,20 +467,21 @@ buildExtractedTarball pkgDir = do
 
 -- | Version of 'checkSDistTarball' that first saves lazy bytestring to
 -- temporary directory and then calls 'checkSDistTarball' on it.
-checkSDistTarball' :: (StackM env m, HasEnvConfig env, MonadBaseUnlift IO m)
+checkSDistTarball'
+  :: HasEnvConfig env
   => SDistOpts
   -> String       -- ^ Tarball name
   -> L.ByteString -- ^ Tarball contents as a byte string
-  -> m ()
+  -> RIO env ()
 checkSDistTarball' opts name bytes = withSystemTempDir "stack" $ \tpath -> do
     npath   <- (tpath </>) `liftM` parseRelFile name
     liftIO $ L.writeFile (toFilePath npath) bytes
     checkSDistTarball opts npath
 
-withTempTarGzContents :: (MonadIO m, MonadMask m)
-  => Path Abs File         -- ^ Location of tarball
-  -> (Path Abs Dir -> m a) -- ^ Perform actions given dir with tarball contents
-  -> m a
+withTempTarGzContents
+  :: Path Abs File                     -- ^ Location of tarball
+  -> (Path Abs Dir -> RIO env a) -- ^ Perform actions given dir with tarball contents
+  -> RIO env a
 withTempTarGzContents apath f = withSystemTempDir "stack" $ \tpath -> do
     archive <- liftIO $ L.readFile (toFilePath apath)
     liftIO . Tar.unpack (toFilePath tpath) . Tar.read . GZip.decompress $ archive
@@ -456,3 +510,17 @@ getModTime :: FilePath -> IO Tar.EpochTime
 getModTime path = do
     t <- getModificationTime path
     return . floor . utcTimeToPOSIXSeconds $ t
+
+getDefaultPackageConfig :: (MonadIO m, MonadReader env m, HasEnvConfig env)
+  => m PackageConfig
+getDefaultPackageConfig = do
+  platform <- view platformL
+  compilerVersion <- view actualCompilerVersionL
+  return PackageConfig
+    { packageConfigEnableTests = False
+    , packageConfigEnableBenchmarks = False
+    , packageConfigFlags = mempty
+    , packageConfigGhcOptions = []
+    , packageConfigCompilerVersion = compilerVersion
+    , packageConfigPlatform = platform
+    }
