@@ -1,17 +1,18 @@
+{-# LANGUAGE NoImplicitPrelude #-}
+{-# LANGUAGE DataKinds #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE DeriveDataTypeable #-}
 {-# LANGUAGE DeriveGeneric #-}
 {-# LANGUAGE FlexibleInstances #-}
+{-# LANGUAGE KindSignatures #-}
+{-# LANGUAGE TypeFamilies #-}
 
 module Stack.Types.Compiler where
 
-import           Control.DeepSeq
 import           Data.Aeson
 import           Data.Data
-import           Data.Monoid ((<>))
-import           Data.Store (Store)
 import qualified Data.Text as T
-import           GHC.Generics (Generic)
+import           Stack.Prelude
 import           Stack.Types.Version
 
 -- | Variety of compiler to use.
@@ -20,6 +21,12 @@ data WhichCompiler
     | Ghcjs
     deriving (Show, Eq, Ord)
 
+-- | Whether the compiler version given is the wanted version (what
+-- the stack.yaml file, snapshot file, or --resolver argument
+-- request), or the actual installed GHC. Depending on the matching
+-- requirements, these values could be different.
+data CVType = CVWanted | CVActual
+
 -- | Specifies a compiler and its version number(s).
 --
 -- Note that despite having this datatype, stack isn't in a hurry to
@@ -28,26 +35,34 @@ data WhichCompiler
 -- NOTE: updating this will change its binary serialization. The
 -- version number in the 'BinarySchema' instance for 'MiniBuildPlan'
 -- should be updated.
-data CompilerVersion
+data CompilerVersion (cvType :: CVType)
     = GhcVersion {-# UNPACK #-} !Version
     | GhcjsVersion
         {-# UNPACK #-} !Version -- GHCJS version
         {-# UNPACK #-} !Version -- GHC version
     deriving (Generic, Show, Eq, Ord, Data, Typeable)
-instance Store CompilerVersion
-instance NFData CompilerVersion
-instance ToJSON CompilerVersion where
+instance Store (CompilerVersion a)
+instance NFData (CompilerVersion a)
+instance ToJSON (CompilerVersion a) where
     toJSON = toJSON . compilerVersionText
-instance FromJSON CompilerVersion where
+instance FromJSON (CompilerVersion a) where
     parseJSON (String t) = maybe (fail "Failed to parse compiler version") return (parseCompilerVersion t)
     parseJSON _ = fail "Invalid CompilerVersion, must be String"
-instance FromJSONKey CompilerVersion where
+instance FromJSONKey (CompilerVersion a) where
     fromJSONKey = FromJSONKeyTextParser $ \k ->
         case parseCompilerVersion k of
             Nothing -> fail $ "Failed to parse CompilerVersion " ++ T.unpack k
             Just parsed -> return parsed
 
-parseCompilerVersion :: T.Text -> Maybe CompilerVersion
+actualToWanted :: CompilerVersion 'CVActual -> CompilerVersion 'CVWanted
+actualToWanted (GhcVersion x) = GhcVersion x
+actualToWanted (GhcjsVersion x y) = GhcjsVersion x y
+
+wantedToActual :: CompilerVersion 'CVWanted -> CompilerVersion 'CVActual
+wantedToActual (GhcVersion x) = GhcVersion x
+wantedToActual (GhcjsVersion x y) = GhcjsVersion x y
+
+parseCompilerVersion :: T.Text -> Maybe (CompilerVersion a)
 parseCompilerVersion t
     | Just t' <- T.stripPrefix "ghc-" t
     , Just v <- parseVersionFromString $ T.unpack t'
@@ -60,27 +75,27 @@ parseCompilerVersion t
     | otherwise
         = Nothing
 
-compilerVersionText :: CompilerVersion -> T.Text
+compilerVersionText :: CompilerVersion a -> T.Text
 compilerVersionText (GhcVersion vghc) =
     "ghc-" <> versionText vghc
 compilerVersionText (GhcjsVersion vghcjs vghc) =
     "ghcjs-" <> versionText vghcjs <> "_ghc-" <> versionText vghc
 
-compilerVersionString :: CompilerVersion -> String
+compilerVersionString :: CompilerVersion a -> String
 compilerVersionString = T.unpack . compilerVersionText
 
-whichCompiler :: CompilerVersion -> WhichCompiler
+whichCompiler :: CompilerVersion a -> WhichCompiler
 whichCompiler GhcVersion {} = Ghc
 whichCompiler GhcjsVersion {} = Ghcjs
 
-isWantedCompiler :: VersionCheck -> CompilerVersion -> CompilerVersion -> Bool
+isWantedCompiler :: VersionCheck -> CompilerVersion 'CVWanted -> CompilerVersion 'CVActual -> Bool
 isWantedCompiler check (GhcVersion wanted) (GhcVersion actual) =
     checkVersion check wanted actual
 isWantedCompiler check (GhcjsVersion wanted wantedGhc) (GhcjsVersion actual actualGhc) =
     checkVersion check wanted actual && checkVersion check wantedGhc actualGhc
 isWantedCompiler _ _ _ = False
 
-getGhcVersion :: CompilerVersion -> Version
+getGhcVersion :: CompilerVersion a -> Version
 getGhcVersion (GhcVersion v) = v
 getGhcVersion (GhcjsVersion _ v) = v
 
