@@ -1,3 +1,4 @@
+{-# LANGUAGE NoImplicitPrelude #-}
 {-# LANGUAGE BangPatterns #-}
 {-# LANGUAGE TemplateHaskell #-}
 {-# LANGUAGE DeriveGeneric #-}
@@ -23,27 +24,17 @@ module Stack.Types.PackageName
   ,packageNameArgument)
   where
 
-import           Control.Applicative
-import           Control.DeepSeq
-import           Control.Monad
-import           Control.Monad.Catch
+import           Stack.Prelude
 import           Data.Aeson.Extended
 import           Data.Attoparsec.Combinators
 import           Data.Attoparsec.Text
-import           Data.Data
-import           Data.Hashable
 import           Data.List (intercalate)
-import           Data.Store (Store)
-import           Data.Text (Text)
 import qualified Data.Text as T
-import           Data.Text.Binary ()
 import qualified Distribution.Package as Cabal
-import           GHC.Generics
 import           Language.Haskell.TH
 import           Language.Haskell.TH.Syntax
 import qualified Options.Applicative as O
 import           Path
-import           Stack.Types.StringError
 
 -- | A parse fail.
 data PackageNameParseFail
@@ -96,7 +87,7 @@ packageNameParser =
 mkPackageName :: String -> Q Exp
 mkPackageName s =
   case parsePackageNameFromString s of
-    Nothing -> errorString ("Invalid package name: " ++ show s)
+    Nothing -> qRunIO $ throwString ("Invalid package name: " ++ show s)
     Just pn -> [|pn|]
 
 -- | Parse a package name from a 'Text'.
@@ -121,15 +112,15 @@ packageNameText (PackageName n) = n
 
 -- | Convert from a Cabal package name.
 fromCabalPackageName :: Cabal.PackageName -> PackageName
-fromCabalPackageName (Cabal.PackageName name) =
-  let !x = T.pack name
+fromCabalPackageName name =
+  let !x = T.pack $ Cabal.unPackageName name
   in PackageName x
 
 -- | Convert to a Cabal package name.
 toCabalPackageName :: PackageName -> Cabal.PackageName
 toCabalPackageName (PackageName name) =
   let !x = T.unpack name
-  in Cabal.PackageName x
+  in Cabal.mkPackageName x
 
 -- | Parse a package name from a file path.
 parsePackageNameFromFilePath :: MonadThrow m => Path a File -> m PackageName
