@@ -1,3 +1,4 @@
+{-# LANGUAGE NoImplicitPrelude #-}
 {-# LANGUAGE ScopedTypeVariables, RankNTypes, DeriveDataTypeable #-}
 
 -- | Run external pagers (@$PAGER@, @less@, @more@) and editors (@$VISUAL@,
@@ -18,18 +19,16 @@ module System.Process.PagerEditor
   ,EditorException(..))
   where
 
-import Control.Exception (try,IOException,throwIO,Exception)
+import Stack.Prelude hiding (ByteString)
 import Data.ByteString.Lazy (ByteString,hPut,readFile)
 import Data.ByteString.Builder (Builder,stringUtf8,hPutBuilder)
-import Data.Typeable (Typeable)
 import System.Directory (findExecutable)
 import System.Environment (lookupEnv)
 import System.Exit (ExitCode(..))
 import System.FilePath ((</>))
 import System.Process (createProcess,shell,proc,waitForProcess,StdStream (CreatePipe)
                       ,CreateProcess(std_in, close_fds, delegate_ctlc))
-import System.IO (hClose,Handle,hPutStr,readFile,withFile,IOMode(WriteMode),stdout)
-import System.IO.Temp (withSystemTempDirectory)
+import System.IO (hPutStr,readFile,stdout)
 
 -- | Run pager, providing a function that writes to the pager's input.
 pageWriter :: (Handle -> IO ()) -> IO ()
