@@ -1,13 +1,15 @@
+{-# LANGUAGE NoImplicitPrelude #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE TemplateHaskell   #-}
 module Stack.Build.TargetSpec (main, spec) where
 
 import qualified Data.Text           as T
 import           Stack.Build.Target
+import           Stack.Prelude
+import           Stack.Types.Config
 import           Stack.Types.PackageIdentifier
 import           Stack.Types.PackageName
 import           Stack.Types.Version
-import           Stack.Types.Package
 import           Test.Hspec
 
 main :: IO ()
