@@ -1,19 +1,16 @@
+{-# LANGUAGE NoImplicitPrelude #-}
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE TupleSections #-}
 -- | Test suite for Stack.Dot
 module Stack.DotSpec where
 
-import           Control.Monad (filterM)
-import           Data.Foldable as F
 import           Data.Functor.Identity
 import           Data.List ((\\))
 import qualified Data.Map as Map
-import           Data.Maybe (fromMaybe)
-import           Data.Set (Set)
 import qualified Data.Set as Set
-import           Data.Text (Text)
 import           Distribution.License (License (BSD3))
+import           Stack.Prelude
 import           Stack.Types.PackageName
 import           Stack.Types.Version
 import           Test.Hspec
@@ -57,14 +54,14 @@ spec = do
 
     prop "requested packages are pruned" $ do
       let resolvedGraph = runIdentity (resolveDependencies Nothing graph stubLoader)
-          allPackages g = Set.map show (Map.keysSet g `Set.union`  F.fold (fmap fst g))
+          allPackages g = Set.map show (Map.keysSet g `Set.union`  fold (fmap fst g))
       forAll (sublistOf (Set.toList (allPackages resolvedGraph))) $ \toPrune ->
         let pruned = pruneGraph [pkgName "one", pkgName "two"] toPrune resolvedGraph
         in Set.null (allPackages pruned `Set.intersection` Set.fromList toPrune)
 
     prop "pruning removes orhpans" $ do
       let resolvedGraph = runIdentity (resolveDependencies Nothing graph stubLoader)
-          allPackages g = Set.map show (Map.keysSet g `Set.union` F.fold (fmap fst g))
+          allPackages g = Set.map show (Map.keysSet g `Set.union` fold (fmap fst g))
           orphans g = Map.filterWithKey (\k _ -> not (graphElem k g)) g
       forAll (sublistOf (Set.toList (allPackages resolvedGraph))) $ \toPrune ->
         let pruned = pruneGraph [pkgName "one", pkgName "two"] toPrune resolvedGraph
